function [MatRLrond] = sub_Lrond(nu,N_d,MatReta,MatRa,MatRg,MatRZ,shss,sh)
    %
    % Copyright C. Soize 10 October 2020
    %
    % MatReta is such that E{RH} = 0 and E{RH RH^T} = [I_nu]. The formulatio in SIAM UQ 2015 (C. Soize) is used
    %
    % for u = u^j with j = 1,...,N_d
    % L = (L_1,...L_nu) (vecteur RL(nu,1)) tel que 
    %      L = - grad_u Phi(u)
    %      grad_u Phi(u) -(1/q(u)) grad_u q(u)
    %      q(u) = 1/N_d Sum_{ell=1}^N_d exp{-||eta^ell - u ||^2 / (2 s^2)}
    %      grad_u q(u) = (1/s^2) 1/N_d Sum_{ell=1}^N_d (eta^ell - u) exp{-||eta^{ell} - u ||^2 / (2 s^2)}
    %
    %---OUTPUT variable
    %         MatRLrond(nu,m) = MatRL(nu,N_d)*MatRa(N_d,m)   

    % s = ((4/((nu+2)*N_d))^(1/(nu+4)));   % Silver bandwidth 
    % s2 = s*s;
    % shss = 1 if icorrectif = 0 and = sqrt(s2+(N_d-1)/N_d) if icorrectif = 1
    % sh = s*shss; 
    %
    %  m is the number of diffusion-maps vectors
    
    sh2 = sh*sh;
    sh2m1 = 1/sh2;
    co1=1/(2*sh2);
    co2=shss/(sh2*N_d);

    MatRU       = MatRZ*MatRg';          % MatRU(nu,N_d), MatRg(N_d,m), MatRZ(nu,m)
    MatRetashss = shss*MatReta;          % MatReta(nu,N_d)
    MatRetaco2  = co2*MatReta;           % MatRetaco2(nu,N_d)

                                                             %---Vectorial sequence without  parallelization
    MatRL = zeros(nu,N_d);                                   % MatRL(nu,N_d)
    for ell=1:N_d
        RU = MatRU(:,ell);                                   % MatRU(nu,N_d)
        MatRexpo   = MatRetashss-repmat(RU,1,N_d);           % MatRexpo(nu,N_d), RU(nu,1)
        Rexpo      = co1*(sum(MatRexpo.^2,1));               % Rexpo(1,N_d) 
        Rexpo_min  = repmat(min(Rexpo),1,N_d);               % Rexpo_min(1,N_d)
        RS         = exp(-(Rexpo-Rexpo_min));                % RS(1,N_d),Rexpo(1,N_d),Rexpo_min(1,N_d)
        q          = sum(RS)/N_d;     
        Rgraduqp   = MatRetaco2*RS';                         % Rgraduqp(nu,1)
        RL         = -sh2m1*RU + Rgraduqp/q;                 % Rgraduqp(nu,1)
        MatRL(:,ell) = RL;                                   % MatRL(nu,N_d)
    end
    MatRLrond = MatRL*MatRa;                                 % MatRLrond(nu,m)
    
    return 
end
