
 function  [epsilonDIFF,mDP,mDMAP,nbmDMAP,Indg,s,shss,sh,Deltar,M0estim,ximean,f0] = ...
                                  sub_diffusion_maps_manifold_for_parallel_parameters(nu,N_d,MatReta_d,ind_generator,icorrectif,Facteur,f0_ref,ind_f0, ...
                                                                                       epsilonDIFFmin) 
%                                             
%-----------------------------------------------------------------------------------------------------------------------------------------
% Copyright C. Soize, 11 October 2020
%------------------------------------------------------------------------------------------------------------------------------------------
%
% estimation of parameters epsilonDIFF, mDP, mDMAP, Indg(nbmDMAP,1)
%--------------------------------------------------------------------------------------------------------------------------------------------
%
%--- INPUT  
%          nu            : dimension or H
%          N_d           : number of realizations in the initial dataset
%          MatReta_d     : MatReta_d(nu,N_d) 
%          indgenerator, icorrectif, Facteur, f0_ref,epsilonDIFFmin
%
%--- OUTPUT
%          epsilonDIFF   :  eps for constructing the kernel in the diffusion maps
%          mDP           :  Maximum number of dominant diffusion-map vectors computed (number of dominant eigenvalues)
%          mDMAP         :  Dimension of the projection such that mDMAP <= mDP
%          nbmDMAP       :  number of DMAP vectors : nbmDMAP <= mDMAP <= mDP
%          Indg          :  Indg(nbmDMAP ,1) contains the indices of the diffusion-map vectors used for the projection of the ISDE    
%          s,shss,sh,Deltar,M0estim,ximean,f0
%
%----------------------------------------------------------------------------------------------------------------------------------- 
                                                                     %--- calculating the parameters 
                                                                     
                                                                         
      s = ((4/((nu+2)*N_d))^(1/(nu+4)));                             % Silver bandwidth          
      modifs = 1;                                                    % in the usual theory, modifs=1;          
      s = modifs*s;                                                  % Silver bandwidth modified 
      s2 = s*s;
      if icorrectif == 0 || icorrectif == 1
         if icorrectif == 0
            shss =1;
         end
         if icorrectif == 1
            shss = 1/sqrt(s2+(N_d-1)/N_d);                           % coef sh/s
         end
      else
          error('STOP: icorrectif must be equal to 0 or equal to 1')
      end      
      sh = s*shss;  
           
      if ind_f0 == 0
         f0 = f0_ref;        
      end
      if ind_f0 == 1
         f0 = f0_ref/sh;         % this the maximum value for f0
      end
      
      Deltar = 2*pi*sh/Facteur;  
      ximean = f0*sh/4;  
                                                                      % estimation  M0estim of the number of sampling points in the transient solution. 
                                                                      % Duration R = M0estim*Deltar for an attenuation of 1/100
      M0estim = 2*log(100)*Facteur/(pi*f0*sh);                        % reference value for a MONTECARLO generation instead of an ERGODIC generation

     if ind_generator == 0
        mDMAP       = N_d;                    % maximum value of mDMAP 
        nbmDMAP     = N_d;                    % number of DMAP vector 
        epsilonDIFF = 0;
        mDP         = 0;
        Indg        = 0;
     end
     
     if ind_generator == 1
      
        iterlimit    = 500;
        
        mDMAP        = nu + 1;                %  maximum value of the numbers of DMAPS basis  
        mDP          = min(mDMAP + 40, N_d);   
        nbmDMAP      = mDMAP - 1;             % number of DMAP vector retained 
        Indg         = (2:1:mDMAP)';          % indices of the DMAP vectors retained 
%         nbmDMAP     = mDMAP ;                % number of DMAP vector retained 
%         Indg        = (1:1:mDMAP)';          % indices of the DMAP vectors retained 
         
        if mDMAP <= 4          
           epsilonDIFF_iterm1 = 0.5;                        % lower bound
           step_epsilonDIFF   = 0.1;
        end
        if mDMAP >=5 && mDMAP <= 10          
           epsilonDIFF_iterm1 = 1;                          % lower bound
           step_epsilonDIFF   = 0.1;
        end
        if mDMAP > 10 && mDMAP <= 20          
           epsilonDIFF_iterm1 = 1;                          % lower bound
           step_epsilonDIFF  = 1;
        end
        if mDMAP > 20          
           epsilonDIFF_iterm1 = epsilonDIFFmin;             % lower bound
           step_epsilonDIFF   = 1 + fix(epsilonDIFFmin/50);
        end
        
        Rcomp = zeros(iterlimit,1);
        
        for iter = 1:iterlimit
         
            epsilonDIFF_iter = epsilonDIFF_iterm1 + step_epsilonDIFF;
            
           [MatRKernel] = sub_Kernel1(N_d,nu,MatReta_d,epsilonDIFF_iter);
            scaling     = max(max(MatRKernel));                                       %--- removing the negative zeros of the eigenvalues of MatRKernel
            MatRKernel  = MatRKernel + scaling*1e-12*eye(N_d,N_d);
                                                                         %--- compute the transition probability matrix MatRP and its symmetrized matrix MatRPs
            MatRPs = zeros(N_d,N_d);                                     %    MatRPs(N_d,N_d)
            Rd   = sum(MatRKernel,2);
            Rdm1s2 = 1./sqrt(Rd);
            for i=1:N_d
                MatRPs(i,:) = Rdm1s2(i)*MatRKernel(i,:).*Rdm1s2';         %    MatRPs(N_d,N_d), Symmetric matrix
            end
            clear MatRKernel scaling Rd  Rdm1s2
            MatRPs = 0.5*(MatRPs+MatRPs');                                %    MatRPs(N_d,N_d), imposing a perfect symmetry
                                       
                                                                          %--- compute the eigenvalues of  MatRPs * MatRphi = MatRphi * diag(Rlambda) 
           if 2*mDP+8 >= N_d                                              % computation with eig
              [~,MatRdiag] =  eig(MatRPs);                                % MatRPs(N_d,N_d)
              Rlambda1   = diag(MatRdiag);                                % Rlambda1(N_d,1)
              [~,Index]  = sort(1./abs(Rlambda1));                        % modulus in descending order
              Rlambda    = Rlambda1(Index);                               % Rlambda(N_d,1)   
              if mDP + 1 <= N_d
                 Rlambda(mDP+1:N_d)   = [];                               % Rlambda(mDP,1)  
              end
           end
           if 2*mDP+8 < N_d                                               % computation with eigs
              OPTS.tol   = 1e-10;                                         % convergence tolerance
              OPTS.p     = 2*mDP+8;                                       % dimension of the Arnoldi basis
              OPTS.maxit = 300;                                           % maximum number of iterations
              OPTS.disp  = 0;                                             % number of eigenvalues displayed at each iteration
              [~,MatRdiag] =  eigs(MatRPs,mDP,'LM',OPTS);          
              Rlambda1 = diag(MatRdiag);                                  % Rlambda1(mDPj,1)
             [~,Index]  = sort(1./abs(Rlambda1));                         % modulus in descending order
              Rlambda = Rlambda1(Index);                                  % Rlambda(mDP,1) 
           end
           clear  MatRdiag MatRPs Rlamnda1 Index 
                                                                           %--- checking the jump in the eigenvalues spectrum
           if mDMAP+1 <= mDP
              comp  = Rlambda(mDMAP+1)/Rlambda(2); 
              compb = Rlambda(mDMAP+1)/Rlambda(mDMAP);
           end
           if mDMAP+1 >  mDP
              comp = Rlambda(mDMAP)/Rlambda(2); 
              compb = comp;
           end
           
           Rcomp(iter) = comp;
    
           if mDMAP <= 20 && (comp <= 0.1  && compb <=0.1)               % solution is epsilonDIFF_iter
              epsilonDIFF  =  epsilonDIFF_iter;
              break
           end
           if mDMAP > 20 && comp <= 0.1                                  % solution is epsilonDIFF_iter
              epsilonDIFF  =  epsilonDIFF_iter;
              break
           end 
           
           if iter == iterlimit   
              if mDMAP+1 <= mDP
                 if mDMAP <= 20
                    h = figure; 
                    plot(Rcomp,'b-o')
                    title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
                    xlabel('iter','FontSize',16,'Interpreter','latex') 
                    ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
                    saveas(h,'figure_STEP3_Rcomp.fig');  
                    close(h);
                    fidlisting=fopen('listing.txt','a+'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'STOP: iterlimit reached without finding a solution'); 
                    fclose(fidlisting);  
                    error('STOP: iterlimit reached without finding a solution')
                 end
                 if mDMAP > 20
                    h = figure; 
                    plot(Rcomp,'b-o')
                    title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
                    xlabel('iter','FontSize',16,'Interpreter','latex') 
                    ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
                    saveas(h,'figure_STEP3_Rcomp.fig');  
                    close(h);
                    fidlisting=fopen('listing.txt','a+'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'STOP: iterlimit reached without finding a solution'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'      Try to increase the value of epsilonDIFFmin'); 
                    fclose(fidlisting);  
                    error('STOP: iterlimit reached without finding a solution')
                 end
              end
              if mDMAP+1 > mDP
                 h = figure; 
                 plot(Rcomp,'b-o')
                 title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
                 xlabel('iter','FontSize',16,'Interpreter','latex') 
                 ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
                 saveas(h,'figure_STEP3_Rcomp.fig');
                 close(h);
                 fidlisting=fopen('listing.txt','a+'); 
                 fprintf(fidlisting,'      \n '); 
                 fprintf(fidlisting,'STOP: iterlimit reached without finding a solution \n'); 
                 fprintf(fidlisting,'      and mDMAP = mDP: diffusionmaps cannot be used \n'); 
                 fprintf(fidlisting,'      re-run with ind_generator = 0'); 
                 fclose(fidlisting);  
                 disp('STOP: iterlimit reached without finding a solution')
                 disp('      and mDMAP = mDP: diffusionmaps cannot be used')
                 disp('      re-run with ind_generator = 0')
                 error('STOP: iterlimit reached without finding a solution') 
              end
           end
           epsilonDIFF_iterm1 = epsilonDIFF_iter;
        end                                        % end loop iter 
        
        h = figure; 
        plot(Rcomp,'b-o')
        title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
        xlabel('iter','FontSize',16,'Interpreter','latex') 
        ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
        saveas(h,'figure_STEP3_Rcomp.fig');  
        close(h);
        
     end                                           % end if ind_generator
      
     return
end
   


 

 