 
 function  [MatReta_argj,RLambdaj] = sub_diffusion_maps_manifold_groupj(N_d,j,mj,MatReta_dj,epsilonDIFFj,tdiffusion,mDPj,mDMAPj,nbmDMAPj,Indgj,f0j, ...
                                                  M0transient,nbMC,NnbMC,ind_generator,shssj,shj,Deltarj,ArrayWiennerM0transientj,ArrayGaussj, ...
                                                  ind_DMAPS,nbmDMAP,MatRg,Rlambda)  
%                                             
%-----------------------------------------------------------------------------------------------------------------------------------------
% Copyright C. Soize, 10 October 2020
%------------------------------------------------------------------------------------------------------------------------------------------
%
% Publications:  C. Soize, R. Ghanem, Data-driven probability concentration and sampling on manifold, 
%                         Journal of Computational Physics,  doi:10.1016/j.jcp.2016.05.044, 321, 242-258 (2016).
%                C. Soize, Optimal partition in terms of independent random vectors of any non-Gaussian vector defined by a set of realizations, 
%                         SIAM-ASA Journal on Uncertainty Quantification}, doi: 10.1137/16M1062223, 5(1), 176-211 (2017).
%                C. Soize, R. Ghanem, Probabilistic learning on manifolds, Foundations of Data Science, 
%                          American  Institute of Mathematical Sciences (AIMS),  doi: 10.3934/fods.2020013, online,  1-29 (2020). 
%--------------------------------------------------------------------------------------------------------------------------------------------
%
%--- INPUT  
%          N_d           : number of realizations in the initial dataset
%          j             : group number
%          mj            : length of vector Y^j = (H_jr1,... ,H_jrmj) of group j
%          MatReta_dj    : MatReta_dj(mj,N_d): realizations of Y^j of length mj
%          epsilonDIFFj  :  eps for constructing the kernel in the diffusion maps
%          tdiffusion    :  tdiffusion = 1 (recommended value).
%          mDPj          :  Maximum number of dominant diffusion-map vectors computed (number of dominant eigenvalues)
%          mDMAPj        :  Dimension of the projection such that mDMAPj <= mDPj
%          nbmDMAPj      :  number of DMAP vectors retained for group j: nbmDMAPj <= mDMAPj <= mDPj
%          Indgj         :  Indg(nbmDMAPj ,1) contains the indices of the diffusion-map vectors used for the projection of the ISDE    
%          f0j           :  Damping coefficient for killing the transient solution (recommended value: f0= 2.5 or 4)
%          M0transient   :  Number of Deltar introduced before the acquisition of the first samping point (recommenden value: M0transient = 100
%          nbMC          :  NnbMC = N_d*nbMC is the number of additional realizations (for instance nbMC = 100) including the initial dataset                        
%          NnbMC         :  NnbMC = N_d*nbMC number of total additional realizations of random vector H that is generated
%                           including the initial dataset. If N_d is large, do not choose a large value of nbMC.
%          ind_generator:      = 0 generator without using the projection of the ISDE on the diffusion maps basis
%                              = 1 generator using the projection of the ISDE on the diffusion maps basis 
%          shssj,shj,Deltarj
%          ArrayWiennerM0transientj(mj,N_d,M0transient,nbMC)
%          ArrayGaussj(mj,N_d,nbMC)
%          ind_DMAPS  = 1 : use of mDMAP and MatRg(nu,mDMAP) constructed in STEP 3 (not mandatory, MatRg(nu,mDMAP) re-constructed in STEP5)
%                     = 2 : use of mDMAPj and MatRgj(mj,mDMAPj) constructed for each group j
%          nbmDMAP            if ind_DMAPS  = 1  and = [] if ind_DMAPS = 2 
%          MatRg(N_d,nbmDMAP) if ind_DMAPS  = 1  and = [] if ind_DMAPS = 2
%          Rlambda(mDP)       if ind_DMAPS  = 1  and = [] if ind_DMAPS = 2
%--- OUTPUT
%          MatReta_argj(mj,NnbMC)
%          RLambdaj(mDPj,1)
%
%----------------------------------------------------------------------------------------------------------------------------------- 
                                                                   
      
%------------------------------------------------------------------------------------------------------------------------------------------  
%                   Stage 1:  computing the diffusion maps  MatRgj(N_d,mDMAPj) associated with MatReta_dj(mj,N_d)
%------------------------------------------------------------------------------------------------------------------------------------------ 

      if ind_generator == 0                                          %--- no projection on the diffusion maps basis
         mDMAPj   = N_d;
         nbmDMAPj = N_d;
         MatRgj   = eye(N_d,N_d);
         RLambdaj = ones(N_d,1);
      end
      
      %--- loading the diffusion maps basis MatRgj(N_d,nbmDMAPj), for each group j, for ind_DMAPS = 1
      if ind_generator == 1 && ind_DMAPS == 1  
         MatRgj   = MatRg;                                            % nbmDMAPj =  nbmDMAP, MatRgj(N_d,nbmDMAPj) = MatRg(N_d,nbmDMAP) 
         RLambdaj = Rlambda;                                          % mDPj = mDP, Rlambdaj(mDPj) = Rlambda(mDP)
      end
      
      %--- constructing the diffusion maps basis MatRgj(N_d,1) = MatReta_dj(1,N_d)' (for group j, for ind_DMAPS = 2 && mj >= 2 , nbmDMAPj = 1) 
      if (ind_generator == 1 && ind_DMAPS == 2) && (mj == 1) 
          mDPj     = 1;
          mDMAPj   = 1;
          nbmDMAPj = 1;
          MatRgj   = MatReta_dj(1,:)';                               % MatRgj(N_d,1), MatReta_dj(1,N_d)
          RLambdaj = ones(mDPj,1);                                   % Rlambdaj(mDPj) = Rlambda(mDP)
      end
      
      %--- constructing the diffusion maps basis MatRgj(N_d,nbmDMAPj), for each group j, for ind_DMAPS = 2 && mj >= 2
      if (ind_generator == 1 && ind_DMAPS == 2) && (mj >=2)                       
        [MatRKernel] = sub_Kernel1(N_d,mj,MatReta_dj,epsilonDIFFj);
                                                                     %    removing the negative zeros of the eigenvalues of MatRKernel
         scaling = max(max(MatRKernel));
         MatRKernel = MatRKernel + scaling*1e-12*eye(N_d,N_d);
                                                                     %    compute the transition probability matrix MatRP and its symmetrized matrix MatRPs
         MatRPs = zeros(N_d,N_d);                                    %    MatRPs(N_d,N_d)
         Rd   = sum(MatRKernel,2);
         Rdm1s2 = 1./sqrt(Rd);
         for i=1:N_d
             MatRPs(i,:) = Rdm1s2(i)*MatRKernel(i,:).*Rdm1s2';       %    MatRPs(N_d,N_d), Symmetric matrix
         end
         clear MatRKernel Rd
         MatRPs = 0.5*(MatRPs+MatRPs');                              %    MatRPs(N_d,N_d), imposing a perfect symmetry
                                           
                                                                     %    compute MatRphi : MatRPs * MatRphi = MatRphi * diag(Rlambda) 

         if 2*mDPj+8 >= N_d                                          % computation with eig
            [MatRphi1,MatRdiag] =  eig(MatRPs);                      % MatRPs(N_d,N_d)
            RLambda1   = diag(MatRdiag);                             % RLambda1(N_d,1)
            [~,Index]  = sort(1./abs(RLambda1));                     % modulus in descending order
            RLambdaj   = RLambda1(Index);                            % RLambdaj(N_d,1)                          
            MatRphi    = MatRphi1(:,Index);                          % MatRphi(N_d,N_d)
            if mDPj + 1 <= N_d
               RLambdaj(mDPj+1:N_d)   = [];                          % RLambdaj(mDPj,1)                          
               MatRphi(:,mDPj+1:N_d) = [];                           % MatRphi(N_d,mDPj)
            end
         end

         if 2*mDPj+8 < N_d                                          % computation with eigs
            OPTS.tol   = 1e-10;                                     % convergence tolerance
            OPTS.p     = 2*mDPj+8;                                  % dimension of the Arnoldi basis
            OPTS.maxit = 300;                                       % maximum number of iterations
            OPTS.disp  = 0;                                         % number of eigenvalues displayed at each iteration
            [MatRphi1,MatRdiag] =  eigs(MatRPs,mDPj,'LM',OPTS);     % MatRphi1(N_d,mDPj)
            RLambda1 = diag(MatRdiag);                              % RLambda1(mDPj,1)
            [~,Index]  = sort(1./abs(RLambda1));                    % modulus in descending order
            RLambdaj  = RLambda1(Index);                            % RLambdaj(mDPj,1)                          
            MatRphi   = MatRphi1(:,Index);                          % MatRphi(N_d,mDPj)
         end
         clear RLambda1 Rbid Index  MatRPhi1 MatRdiag
         clear MatRPs 
                                                                    % Plot
         hold off 
         h=figure;                                
         semilogy((1:1:mDPj)',RLambdaj(1:1:mDPj),'-o') 
         title(['Eigenvalues $\Lambda_{',num2str(j),',\alpha}$ of the transition matrix in $\log_{10}$ scale'],'FontSize',16,'Interpreter','latex')                                         
         xlabel('$\alpha$','FontSize',16,'Interpreter','latex')                                                                
         ylabel(['$\Lambda_{',num2str(j),',\alpha}$'],'FontSize',16,'Interpreter','latex')
         saveas(h,['figure_STEP5_eigenvalueDMAPS_group_',num2str(j),'.fig']); 
         hold off
         close(h)
                                                                    % compute MatRPsi : MatRP * MatRPsi = MatRPsi * diag(Rlambda)
         MatRPsi = zeros(N_d,mDPj);                            
         for k=1:mDPj
             MatRPsi(:,k) = MatRphi(:,k).*Rdm1s2;        
         end  
         RLambdajt = RLambdaj.^tdiffusion; 
         clear MatRphi Rdm1s2
                                                                  % loading vectors of the diffusion maps
         MatRgj = zeros(N_d,nbmDMAPj);                            % MatRgj(N_d,nbmDMAPj) selecting the nbmDMAPj diffusion maps with Indgj(1),...,Indgj(nbmDMAPj)      
         for beta=1:nbmDMAPj                                       
             MatRgj(:,beta) = RLambdajt(Indgj(beta))*MatRPsi(:,Indgj(beta)); % MatRgj(N_d,nbmDMAPj) 
         end
         clear RLambdat MatRPsi                             
      end                            %--- end if indgenerator = 1 && ind_DMAPS = 2
      
      
     %--- generation of MatRaj(N_d,nbmDMAPj)   
      MatRaj = MatRgj/(MatRgj'*MatRgj);  % MatRaj(N_d,nbmDMAPj): [a] =[g]([g]'[g])^(-1) 
        
%------------------------------------------------------------------------------------------------------------------------------------------  
%                   Stage 2:  generation  of NnbMC realizations  MatReta_argj(mj,NnbMC) loaded in ArrayH_argj(mj,N_d,nbMC)  
%------------------------------------------------------------------------------------------------------------------------------------------                                               

                                             %--- solving the Ito stochastic differential equation in MatRZ
                                             %    ArrayH_argj(nu,N_d,nbMC) = [MatReta_argj(mj,N_d)^1 ... MatReta_argj(mj,N_d)^nbMC)] 
                                                             
                                             %    ArrayZ_argj(mj,nbmDMAPj,nbMC)
                                             %    ArrayWiennerM0transientj(mj,N_d,M0transient,nbMC),MatRGaussj(mj,N_d,nbMC) 
                                                            
      ArrayZ_argj = zeros(mj,nbmDMAPj,nbMC);                                    %    ArrayZ_ar(mj,nbmDMAPj,nbMC)
      
      parfor ell = 1:nbMC 
         MatRGaussj_ell               = ArrayGaussj(:,:,ell);                   %  MatRGaussj_ell(mj,N_d), ArrayGaussj(mj,N_d,nbMC)
         ArrayWiennerM0transientj_ell = ArrayWiennerM0transientj(:,:,:,ell);    %  ArrayWiennerM0transientj_ell(mj,N_d,M0transient),ArrayWiennerM0transientj(mj,N_d,M0transient,nbMC)
         
         [MatRZ_arg] = sub_integration_Verlet_MC_for_parallel(ell,mj,nbmDMAPj,N_d,M0transient,Deltarj,f0j,MatReta_dj,MatRaj,MatRgj,shssj,shj, ...
                                                              ArrayWiennerM0transientj_ell,MatRGaussj_ell);                                                    
                                                             
          ArrayZ_argj(:,:,ell) = MatRZ_arg;                                   
      end
      clear MatRZ_arg MatRGaussj_ell ArrayWiennerM0transientj_ell                                     
                                                                  %--- computing ArrayH_argj(mj,N_d,nbMC)
      ArrayH_argj = zeros(mj,N_d,nbMC);                                           
      for ell=1:nbMC
             ArrayH_argj(:,:,ell) = ArrayZ_argj(:,:,ell)*MatRgj'; %  ArrayH_argj(mj,N_d,nbMC), ArrayZ_argj(mj,nbmDMAPj,nbMC), MatRgj(N_d,nbmDMAPj) 
      end
     
      MatReta_argj  = reshape(ArrayH_argj,mj,NnbMC);              %    MatReta_argj(mj,NnbMC) 
      clear ArrayH_argj
      
  return
end
   


 

 