
 function  [MatReta_argj,RLambdaj] = sub_diffusion_maps_manifold_groupj_c(N_d,j,mj,MatReta_dj,epsilonDIFFj,tdiffusion,mDPj,mDMAPj,nbmDMAPj,Indgj,f0j, ...
                                                  M0transient,nbMC,NnbMC,ind_generator,shssj,shj,Deltarj,ArrayWiennerM0transientj,ArrayGaussj, ...
                                                  ind_constraints,ind_coupling,epsc,iter_limit,Ralpha_relax,minVarH,maxVarH, ...
                                                  ind_DMAPS,nbmDMAP,MatRg,Rlambda)  
%                                             
%-----------------------------------------------------------------------------------------------------------------------------------------
% Copyright C. Soize, 09 November 2020
%------------------------------------------------------------------------------------------------------------------------------------------
%
% Publications:  C. Soize, R. Ghanem, Data-driven probability concentration and sampling on manifold, 
%                         Journal of Computational Physics,  doi:10.1016/j.jcp.2016.05.044, 321, 242-258 (2016).
%                C. Soize, Optimal partition in terms of independent random vectors of any non-Gaussian vector defined by a set of realizations, 
%                         SIAM-ASA Journal on Uncertainty Quantification}, doi: 10.1137/16M1062223, 5(1), 176-211 (2017).
%                C. Soize, R. Ghanem, Probabilistic learning on manifolds, Foundations of Data Science, 
%                          American  Institute of Mathematical Sciences (AIMS),  doi: 10.3934/fods.2020013, online,  1-29 (2020). 
%                C. Soize, R. Ghanem, Physics-constrained non-Gaussian probabilistic learning on manifolds, 
%                          International Journal for Numerical Methods in Engineering, doi: 10.1002/nme.6202, 121 (1), 110-145 (2020).
%--------------------------------------------------------------------------------------------------------------------------------------------
%
%--- INPUT  
%          N_d           : number of realizations in the initial dataset
%          j             : group number
%          mj            : length of vector Y^j = (H_jr1,... ,H_jrmj) of group j
%          MatReta_dj    : MatReta_dj(mj,N_d): realizations of Y^j of length mj
%          epsilonDIFFj  :  eps for constructing the kernel in the diffusion maps
%          tdiffusion    :  tdiffusion = 1 (recommended value).
%          mDPj          :  Maximum number of dominant diffusion-map vectors computed (number of dominant eigenvalues)
%          mDMAPj        :  Dimension of the projection such that mDMAPj <= mDPj
%          nbmDMAPj      :  number of DMAP vectors retained for group j: nbmDMAPj <= mDMAPj <= mDPj
%          Indgj         :  Indg(nbmDMAPj ,1) contains the indices of the diffusion-map vectors used for the projection of the ISDE    
%          f0j           :  Damping coefficient for killing the transient solution (recommended value: f0= 2.5 or 4)
%          M0transient   :  Number of Deltar introduced before the acquisition of the first samping point (recommenden value: M0transient = 100
%          nbMC          :  NnbMC = N_d*nbMC is the number of additional realizations (for instance nbMC = 100) including the initial dataset                        
%          NnbMC         :  NnbMC = N_d*nbMC number of total additional realizations of random vector H that is generated
%                           including the initial dataset. If N_d is large, do not choose a large value of nbMC.
%          ind_generator:      = 0 generator without using the projection of the ISDE on the diffusion maps basis
%                              = 1 generator using the projection of the ISDE on the diffusion maps basis 
%          shssj,shj,Deltarj
%          ArrayWiennerM0transientj(mj,N_d,M0transient,nbMC)
%          ArrayGaussj(mj,N_d,nbMC)
%          ind_constraints = 0 : no constraints E{H] = 0 and E{H H'} = [I_nu]
%                          = 1 : constraints E{H_j^2} = 1 for j =1,...,nu   
%                          = 2 : constraints E{H] = 0 and E{H_j^2} = 1 for j =1,...,nu
%                          = 3 : constraints E{H] = 0 and E{H H'} = [I_nu]  (HIGHLY RECOMMENDED; if problems, use ind_constraints=1)                                
%                                   --- parameters for the convergence of the iteration algorithm if ind_constraints >= 1
%           ind_coupling  = 0;      % = 0 : for ind_constraints = 2 or 3, no coupling in  matrix MatRGammaS_iter (HIGHLY RECOMMENDED)
%                                   % = 1 : for ind_constraints = 2 or 3, coupling,  all the extra-diagonal blocs in matrix MatRGammaS_iter are kept
%           epsc          = 1.2e-4; %    relative tolerance for the iteration convergence in the constraints E{H} = 0 and E{H H'} = [I_nu] 
%           iter_limit    = 100;    %    maximum number of iteration for computing the Lagrange multiplier
%                                   %... Defining the relaxation parameter Ralpha_relax_step5(iter_limit,1)
%           alpha_relax1   = 0.01;  %    value of alpha_relax for iter = 1  
%           iter_relax2    = 20;    %    value of iter for which alpha_relax = alpha_relax2; if iter_relax2 = 1, then alpha_relax = alpha_relax2 for iter >=1   
%           alpha_relax2   = 1.0;   %    value of alpha_relax for iter >= iter_relax2 
                                    %    NOTE 1: IF  iter_relax2 = 1 , THEN: Ralpha_relax_step5(iter) = alpha_relax2 for iter >=1
                                    %    NOTE 2: IF  iter_relax2 >= 2, THEN  
                                    %                        for iter >= 1 and for iter < iter_relax2, we have:
                                    %                            Ralpha_relax_step5(iter) = alpha_relax1 +(alpha_relax2 - alpha_relax1)*(iter-1)/(iter_relax2-1);
                                    %                        for iter >= iter_relax2, we have:
                                    %                            Ralpha_relax_step5(iter) = alpha_relax2
                                    %    NOTE 3: for decreasing the error err(iter), increase the value of iter_relax2
                                    %    NOTE 4: if divergence during ITO solver, decrease alpha_relax2 or increase iter_relax2
%          minVarH          = minimum imposed on Var{H^2} with respect to 1 (for instance 0.99) for the convergence test
%          maxVarH          = maximum imposed on Var{H^2} with respect to 1 (for instance 1.01) for the convergence test  
%          ind_DMAPS  = 1 : use of mDMAP and MatRg(nu,mDMAP) constructed in STEP 3 (not mandatory, MatRg(nu,mDMAP) re-constructed in STEP5)
%                     = 2 : use of mDMAPj and MatRgj(mj,mDMAPj) constructed for each group j
%          nbmDMAP            if ind_DMAPS  = 1  and = [] if ind_DMAPS = 2 
%          MatRg(N_d,nbmDMAP) if ind_DMAPS  = 1  and = [] if ind_DMAPS = 2
%          Rlambda(mDP)       if ind_DMAPS  = 1  and = [] if ind_DMAPS = 2
%
%--- OUTPUT
%          MatReta_argj(mj,NnbMC)
%          RLambdaj(mDPj,1)
%
%----------------------------------------------------------------------------------------------------------------------------------- 
                                                                   
      
%------------------------------------------------------------------------------------------------------------------------------------------  
%                   Stage 1:  computing the diffusion maps  MatRg(N_d,mDMAPj) associated with MatReta_dj(mj,N_d)
%------------------------------------------------------------------------------------------------------------------------------------------ 

      if ind_generator == 0                                          %--- no projection on the diffusion maps basis
         mDMAPj   = N_d;
         nbmDMAPj = N_d;
         MatRgj   = eye(N_d,N_d);
         RLambdaj = ones(N_d,1);
      end
      
      %--- loading the diffusion maps basis MatRgj(N_d,nbmDMAPj) for each group j for ind_DMAPS = 1
      if ind_generator == 1 && ind_DMAPS == 1  
         MatRgj   = MatRg;                                            % nbmDMAPj =  nbmDMAP, MatRgj(N_d,nbmDMAPj) = MatRg(N_d,nbmDMAP) 
         RLambdaj = Rlambda;                                          % mDPj = mDP, Rlambdaj(mDPj) = Rlambda(mDP)
      end
      
      %--- constructing the diffusion maps basis MatRgj(N_d,1) = MatReta_dj(1,N_d)' (for group j, for ind_DMAPS = 2 && mj >= 2 , nbmDMAPj = 1) 
      if (ind_generator == 1 && ind_DMAPS == 2) && (mj == 1) 
          mDPj     = 1;
          mDMAPj   = 1;
          nbmDMAPj = 1;
          MatRgj   = MatReta_dj(1,:)';                               % MatRgj(N_d,1), MatReta_dj(1,N_d)
          RLambdaj = ones(mDPj,1);                                   % Rlambdaj(mDPj) = Rlambda(mDP)
      end
      
      %--- constructing the diffusion maps basis MatRgj(N_d,nbmDMAPj), for each group j, for ind_DMAPS = 2 && mj >= 2
      if (ind_generator == 1 && ind_DMAPS == 2) && (mj >=2)      
        [MatRKernel] = sub_Kernel1(N_d,mj,MatReta_dj,epsilonDIFFj);
                                                                     %    removing the negative zeros of the eigenvalues of MatRKernel
         scaling = max(max(MatRKernel));
         MatRKernel = MatRKernel + scaling*1e-12*eye(N_d,N_d);
                                                                     %    compute the transition probability matrix MatRP and its symmetrized matrix MatRPs
         MatRPs = zeros(N_d,N_d);                                    %    MatRPs(N_d,N_d)
         Rd   = sum(MatRKernel,2);
         Rdm1s2 = 1./sqrt(Rd);
         for i=1:N_d
             MatRPs(i,:) = Rdm1s2(i)*MatRKernel(i,:).*Rdm1s2';       %    MatRPs(N_d,N_d), Symmetric matrix
         end
         clear MatRKernel Rd
         MatRPs = 0.5*(MatRPs+MatRPs');                              %    MatRPs(N_d,N_d), imposing a perfect symmetry
                                           
                                                                     %    compute MatRphi : MatRPs * MatRphi = MatRphi * diag(Rlambda) 

         if 2*mDPj+8 >= N_d                                          % computation with eig
            [MatRphi1,MatRdiag] =  eig(MatRPs);                      % MatRPs(N_d,N_d)
            RLambda1   = diag(MatRdiag);                             % RLambda1(N_d,1)
            [~,Index]  = sort(1./abs(RLambda1));                     % modulus in descending order
            RLambdaj   = RLambda1(Index);                            % RLambdaj(N_d,1)                          
            MatRphi    = MatRphi1(:,Index);                          % MatRphi(N_d,N_d)
            if mDPj + 1 <= N_d
               RLambdaj(mDPj+1:N_d)   = [];                          % RLambdaj(mDPj,1)                          
               MatRphi(:,mDPj+1:N_d) = [];                           % MatRphi(N_d,mDPj)
            end
         end

         if 2*mDPj+8 < N_d                                          % computation with eigs
            OPTS.tol   = 1e-10;                                     % convergence tolerance
            OPTS.p     = 2*mDPj+8;                                   % dimension of the Arnoldi basis
            OPTS.maxit = 300;                                       % maximum number of iterations
            OPTS.disp  = 0;                                         % number of eigenvalues displayed at each iteration
            [MatRphi1,MatRdiag] =  eigs(MatRPs,mDPj,'LM',OPTS);     % MatRphi1(N_d,mDPj)
            RLambda1 = diag(MatRdiag);                              % RLambda1(mDPj,1)
           [~,Index]  = sort(1./abs(RLambda1));                     % modulus in descending order
            RLambdaj  = RLambda1(Index);                            % RLambdaj(mDPj,1)                          
            MatRphi   = MatRphi1(:,Index);                          % MatRphi(N_d,mDPj)
         end
         clear RLambda1 Rbid Index  MatRPhi1 MatRdiag
         clear MatRPs 
                                                                    % Plot
         hold off 
         h=figure;                                
         semilogy((1:1:mDPj)',RLambdaj(1:1:mDPj),'-o') 
         title(['Eigenvalues $\Lambda_{',num2str(j),',\alpha}$ of the transition matrix in $\log_{10}$ scale'],'FontSize',16,'Interpreter','latex')                                         
         xlabel('$\alpha$','FontSize',16,'Interpreter','latex')                                                                
         ylabel(['$\Lambda_{',num2str(j),',\alpha}$'],'FontSize',16,'Interpreter','latex')
         saveas(h,['figure_STEP5_eigenvalueDMAPS_group_',num2str(j),'.fig']); 
         hold off
         close(h)
                                                                    % compute MatRPsi : MatRP * MatRPsi = MatRPsi * diag(Rlambda)
         MatRPsi = zeros(N_d,mDPj);                            
         for k=1:mDPj
             MatRPsi(:,k) = MatRphi(:,k).*Rdm1s2;        
         end  
         RLambdajt = RLambdaj.^tdiffusion; 
         clear MatRphi Rdm1s2
                                                                  % loading vectors of the diffusion maps
         MatRgj = zeros(N_d,nbmDMAPj);                            % MatRgj(N_d,nbmDMAPj) selecting the nbmDMAPj diffusion maps with Indgj(1),...,Indgj(nbmDMAPj)      
         for beta=1:nbmDMAPj                                       
             MatRgj(:,beta) = RLambdajt(Indgj(beta))*MatRPsi(:,Indgj(beta)); % MatRgj(N_d,nbmDMAPj) 
         end
         clear RLambdat MatRPsi                             
      end                             %--- end if indgenerator = 1 && ind_DMAPS = 2
      
      %--- generation of MatRaj(N_d,nbmDMAPj) 
      MatRaj = MatRgj/(MatRgj'*MatRgj);  % MatRaj(N_d,nbmDMAPj): [a] =[g]([g]'[g])^(-1) 
        
%------------------------------------------------------------------------------------------------------------------------------------------  
%                   Stage 2:  generation  of NnbMC realizations  MatReta_argj(mj,NnbMC) loaded in ArrayH_argj(mj,N_d,nbMC)  
%------------------------------------------------------------------------------------------------------------------------------------------                                               
      disp(' ')
      disp('============================= Group j = '); disp(j)
      
      if ind_constraints == 1                   %--- constraints E{Hj_k^2} = 1 for k =1,...,mj  
         mhcj = mj;
         Rbcj = ones(mj,1);                     %    Rbcj(mhcj,1) :  E{h^cj(Hj)} = b^cj corresponding to  E{Hj_k^2} = 1 for k =1,...,mj
         Rlambdaj_iter_m1 = zeros(mhcj,1);      %    Rlambdaj_iter_m1(mhcj,1) = ( lambdaj_1,...,lambdaj_mj )
         MatRlambdaj_iter_m1 =[]; 
      end
      if ind_constraints == 2                   %--- constraints E{Hj_k} = 0 for k =1,...,mj and E{Hj_k^2} = 1 for k =1,...,mj
         mhcj = 2*mj;
         Rbcj = zeros(mhcj,1);                  %    Rbcj(mhcj,1) :  E{h^cj(Hj)} = b^cj corresponding to E{Hj_k} = 0 for k =1,...,mj and E{Hj_k^2} = 1 for k =1,...,mj
         Rbcj(1+mj:2*mj,1) = ones(mj,1);  
         Rlambdaj_iter_m1 = zeros(mhcj,1);      %    Rlambdaj_iter_m1(mhcj,1) = (lambdaj_1,...,lambdaj_mj,lambdaj_{1+mj},...,lambdaj_{mj+mj})
         MatRlambdaj_iter_m1 =[];      
      end
      if ind_constraints == 3                   %--- constraints E{H] = 0 and E{H H'} = [I_nu]
         mhcj = 2*mj + mj*(mj-1)/2;             %    note that for mj = 1, we have mhcj = 2*mj
                                                %    if mj = 1: Rbcj(mhcj,1) = (bcj_1,...,bcj_mj , bcj_{1+mj},...,bcj_{mj+mj} )
                                                %    if mj > 1:  Rbcj(mhcj,1) = (bcj_1,...,bcj_mj , bcj_{1+mj},...,bcj_{mj+mj}, {bcj_{(jj,ii) + 2mj} for 1 <= jj < ij <= nu} )
         Rbcj = zeros(mhcj,1);                  %    Rbcj(mhcj,1) :  E{h^cj(Hj)} = b^cj corresponding to E{Hj] = 0  and  E{Hj Hj'} = [I_mj] 
         Rbcj(1+mj:2*mj,1) = ones(mj,1);
         Rlambdaj_iter_m1  = zeros(mhcj,1);     %    Rlambdaj_iter_m1(mhcj,1)      
         if mj == 1                             %    if mj = 1: Rlambdaj_iter(mhcj,1) = (lambdaj_1,...,lambdaj_mj,lambdaj_{1+mj},...,lambdaj_{mj+mj})
            MatRlambdaj_iter_m1 =[];
         end    
         if mj >=2                              %    if mj > 1: Rlambdaj_iter(mhcj,1) = ( lambdaj_1,...,lambdaj_mj , lambdaj_{1+mj},...,lambdaj_{mj+mj} ,
                                                %                                      {lambdaj_{(jj,ii) + 2 mj} for 1 <= jj < ii <= mj} )
            MatRlambdaj_iter_m1 = zeros(mj,mj); %    MatRlambdaj_iter_m1(mj,mj) = Symmetric with zero diagonal and upper part is  Rlambdaj_iter  row rise
            ind = 0;
            for jj = 1:mj-1
                for ii = jj+1:mj
                    ind = ind + 1;
                    MatRlambdaj_iter_m1(jj,ii) = Rlambdaj_iter_m1(2*mj+ind);
                    MatRlambdaj_iter_m1(ii,jj) = MatRlambdaj_iter_m1(jj,ii);
                end
            end
        end
      end
      
      fidlisting=fopen('listing.txt','a+'); 
      fprintf(fidlisting,'      \n '); 
      fprintf(fidlisting,'--------- Group number   = %7i \n ',j); 
      fprintf(fidlisting,'                      \n '); 
      fprintf(fidlisting,'          mhcj           = %7i \n ',mhcj); 
      fprintf(fidlisting,'                      \n '); 
      fclose(fidlisting);  
      disp(' ')
      disp(' mj   = '); disp(mj) 
      disp(' mhcj = '); disp(mhcj)  
      
      %--- Computation of the Lagrange multipliers by the iteration algorithm  and generation of nbMC realizations 
                                                          
      Rerrj           = zeros(iter_limit,1);   %    Rerrj(iter_limit,1)      
      RnormRlambdaj   = zeros(iter_limit,1);   %    RnormRlambdaj(iter_limit,1)
      RcondGammaSj    = zeros(iter_limit,1);   %    RcondGammaSj(iter_limit,1) 
      Rtolj           = zeros(iter_limit,1);   %    Rtolj(iter_limit,1)  
      Rtolj(1)        = 1;
      ind_convj       = 0;
            
      for iter = 1:iter_limit                              %    LOOP of the iteration algorithm (iter : lambdaj_iter given and compute lambdaj_iterplus1
                                                           %---------------------------------------------------------------------------------------------
          disp('-------------iter'); disp(iter)            %    For lambdaj = lambdaj_iter, solving the Ito stochastic differential equation in MatRZ
                                                           %    ArrayH_argj(mj,N_d,nbMC) = [MatReta_argj(mj,N_d)^1 ... MatReta_argj(mj,N_d)^nbMC)] 
                                                           %    with  MatReta_argj(mj,N_d)^ell the realization for ell = 1,...,nbMC   
                                                           %--- Computing additional realizations 
                                                           %    MatReta_argj_iter(mj,nbMC) 
          iter_plot  = iter;                                                 
                                                           %--- solving the Ito stochastic differential equation in  ArrayZ_argj_iter(mj,nbmDMAPj,nbMC) 
                                                           
          if ind_constraints == 1                          %--- constraints E{Hj_k^2} = 1 for k =1,...,mj  
             ArrayZ_argj_iter = zeros(mj,nbmDMAPj,nbMC);   %    ArrayZ_argj_iter(mj,nbmDMAPj,nbMC)                                                      

             parfor ell = 1:nbMC 
                 MatRGaussj_ell               = ArrayGaussj(:,:,ell);                % MatRGaussj_ell(mj,N_d), ArrayGaussj(mj,N_d,nbMC)
                 ArrayWiennerM0transientj_ell = ArrayWiennerM0transientj(:,:,:,ell); % ArrayWiennerM0transientj_ell(mj,N_d,M0transient),ArrayWiennerM0transientj(mj,N_d,M0transient,nbMC)

                 [MatRZ_arg] = sub_integration_Verlet_MC_for_parallel_c1(ell,mj,nbmDMAPj,N_d,M0transient,Deltarj,f0j,MatReta_dj,MatRaj,MatRgj,shssj,shj, ...
                                                                         ArrayWiennerM0transientj_ell,MatRGaussj_ell,mhcj,Rlambdaj_iter_m1);  
                 ArrayZ_argj_iter(:,:,ell) = MatRZ_arg;                                   
             end
             clear MatRZ_arg MatRGaussj_ell ArrayWiennerM0transientj_ell 
          end
           
          if ind_constraints == 2                          %--- constraints  E{Hj_k} = 0 for k =1,...,mj and E{Hj_k^2} = 1 for k =1,...,mj
             ArrayZ_argj_iter = zeros(mj,nbmDMAPj,nbMC);   %    ArrayZ_argj_iter(mj,nbmDMAPj,nbMC)                                                      

             parfor ell = 1:nbMC 
                 MatRGaussj_ell               = ArrayGaussj(:,:,ell);                % MatRGaussj_ell(mj,N_d), ArrayGaussj(mj,N_d,nbMC)
                 ArrayWiennerM0transientj_ell = ArrayWiennerM0transientj(:,:,:,ell); % ArrayWiennerM0transientj_ell(mj,N_d,M0transient),ArrayWiennerM0transientj(mj,N_d,M0transient,nbMC)

                 [MatRZ_arg] = sub_integration_Verlet_MC_for_parallel_c2(ell,mj,nbmDMAPj,N_d,M0transient,Deltarj,f0j,MatReta_dj,MatRaj,MatRgj,shssj,shj, ...
                                                                         ArrayWiennerM0transientj_ell,MatRGaussj_ell,mhcj,Rlambdaj_iter_m1);  
                 ArrayZ_argj_iter(:,:,ell) = MatRZ_arg;                                   
             end
             clear MatRZ_arg MatRGaussj_ell ArrayWiennerM0transientj_ell 
          end
          
          if ind_constraints == 3                          %--- constraints E{Hj] = 0 and E{Hj Hj'} = [I_mj] 
             ArrayZ_argj_iter = zeros(mj,nbmDMAPj,nbMC);   %    ArrayZ_argj_iter(mj,nbmDMAPj,nbMC)                                                      

             parfor ell = 1:nbMC 
                 MatRGaussj_ell               = ArrayGaussj(:,:,ell);                % MatRGaussj_ell(mj,N_d), ArrayGaussj(mj,N_d,nbMC)
                 ArrayWiennerM0transientj_ell = ArrayWiennerM0transientj(:,:,:,ell); % ArrayWiennerM0transientj_ell(mj,N_d,M0transient),ArrayWiennerM0transientj(mj,N_d,M0transient,nbMC)

                 [MatRZ_arg] = sub_integration_Verlet_MC_for_parallel_c3(ell,mj,nbmDMAPj,N_d,M0transient,Deltarj,f0j,MatReta_dj,MatRaj,MatRgj,shssj,shj, ...
                                                                         ArrayWiennerM0transientj_ell,MatRGaussj_ell,mhcj,Rlambdaj_iter_m1,MatRlambdaj_iter_m1);  
                 ArrayZ_argj_iter(:,:,ell) = MatRZ_arg;                                   
             end
             clear MatRZ_arg MatRGaussj_ell ArrayWiennerM0transientj_ell 
          end
          
          ArrayH_argj_iter = zeros(mj,N_d,nbMC);                             %--- computing ArrayH_argj_iter(mj,N_d,nbMC)                                         
          for ell=1:nbMC
              ArrayH_argj_iter(:,:,ell) = ArrayZ_argj_iter(:,:,ell)*MatRgj'; % ArrayH_argj_iter(mj,N_d,nbMC), ArrayZ_argj_iter(mj,nbmDMAPj,nbMC), MatRgj(N_d,nbmDMAPj) 
          end
                                                                             %--- computing hcj(Hj) loaded in MatRhcj_iter(mhcj,NnbMC)
          MatReta_argj_iter  = reshape(ArrayH_argj_iter,mj,NnbMC);           %    MatReta_argj_iter(mj,NnbMC)
          clear ArrayH_argj_iter   
          
          normetaarj = norm(MatReta_argj_iter,'fro');
          testNaN = isnan(normetaarj); % = 1 if NaN, and = 0 if not
          testInf = isinf(normetaarj); % = 1 if +Inf of -Inf, and = 0 if not            
          if testNaN  >= 1  ||  testInf  >= 1                                                       
             disp('----- STOP: indetermination of type NaN of Inf in the ISDE solver ')
             disp('iter'); disp(iter);
             disp('Rlambdaj_iter_m1');disp(Rlambdaj_iter_m1');
             disp('            Decrease alpha_relax2 or increase iter_relax2 ');
             fidlisting=fopen('listing.txt','a+');
             fprintf(fidlisting,'      \n ');
             fprintf(fidlisting,'      \n ');
             fprintf(fidlisting,'----- STOP: ndetermination of type 0/0 in the ISDE solver \n '); 
             fprintf(fidlisting,'      \n '); 
             fprintf(fidlisting,'      \n ');             
             fprintf(fidlisting,'       iter             = %7i \n ',iter);
             fprintf(fidlisting,'       Rlambdaj_iter_m1  = %14.7e %14.7e %14.7e %14.7e %14.7e %14.7e %14.7e %14.7e \n ',Rlambdaj_iter_m1');
             fprintf(fidlisting,'      \n ');
             fprintf(fidlisting,'       --- Decrease alpha_relax2 or increase iter_relax2\n ');
             fprintf(fidlisting,'      \n ');
             fclose(fidlisting); 
             error('STOP')                                                   %     STOP: divergence of ISDE
          end
          if ind_constraints == 1                                            %--- constraints E{Hj_k^2} = 1 for k =1,...,mj
             MatRhcj_iter              = zeros(mhcj,NnbMC);                  %    MatRhcj_iter(mhcj,NnbMC);
             MatRhcj_iter(1:mj,:)      = MatReta_argj_iter.^2;               %    MatReta_argj_iter(mj,NnbMC),MatRhcj_iter(mhcj,NnbMC)
          end
          if ind_constraints == 2                                            %--- constraints E{Hj_k} = 0 for k =1,...,mj and E{Hj_k^2} = 1 for k =1,...,mj
             MatRhcj_iter              = zeros(mhcj,NnbMC);                  %    MatRhcj_iter(mhcj,NnbMC);
             MatRhcj_iter(1:mj,:)      = MatReta_argj_iter;                  %    MatReta_argj_iter(mj,NnbMC),MatRhcj_iter(mhcj,NnbMC)
             MatRhcj_iter(1+mj:2*mj,:) = MatReta_argj_iter.^2;               %    MatReta_argj_iter(mj,NnbMC),MatRhcj_iter(mhcj,NnbMC)
          end
          if ind_constraints == 3                                            %--- constraints E{Hj] = 0 and E{Hj Hj'} = [I_mj]
             MatRhcj_iter              = zeros(mhcj,NnbMC);                  %    MatRhcj_iter(mhcj,NnbMC);
             MatRhcj_iter(1:mj,:)      = MatReta_argj_iter;                  %    MatReta_argj_iter(mj,NnbMC),MatRhcj_iter(mhcj,NnbMC)
             MatRhcj_iter(1+mj:2*mj,:) = MatReta_argj_iter.^2;               %    MatReta_argj_iter(mj,NnbMC),MatRhcj_iter(mhcj,NnbMC)
             if mj >=2  
                ind = 0;
                for jj = 1:mj-1
                    for ii = jj+1:mj
                        ind = ind + 1;
                        MatRhcj_iter(2*mj+ind,:) = MatReta_argj_iter(jj,:).*MatReta_argj_iter(ii,:);
                    end
                end
             end
          end
          RVarHj_iter          = var(MatReta_argj_iter,0,2);
          minVarHj_iter        = min(RVarHj_iter );
          maxVarHj_iter        = max(RVarHj_iter );
          clear RVarHj_iter
                                                  
          Rmeanhcj_iter          = mean(MatRhcj_iter,2);                         %  Rmeanhcj_iter(mhcj,1),MatRhcj_iter(mhcj,NnbMC)          
          RGammaPj_iter          = Rbcj - Rmeanhcj_iter;                         %  RGammaPj_iter(mhcj,1),Rbcj(mhcj,1),Rmeanhcj_iter(mhcj,1) 
          MatRGammaSj_iter_temp  = cov(MatRhcj_iter');                           %  MatRGammaSj_iter_temp(mhcj,mhcj) 
          
          if ind_coupling == 0                                                   %--- For ind_constraints = 2 or 3, no coupling between the Lagrange 
                                                                                 %    multipliers in matrix MatRGammaS_iter 
             if ind_constraints == 1                                             %... constraints E{Hj_k^2} = 1 for k =1,...,mj
                 MatRGammaSj_iter  = MatRGammaSj_iter_temp;                      %    MatRGammaSj_iter(mhcj,mhcj)   
             end
             if ind_constraints == 2                                             %... constraints E{Hj_k} = 0 for k =1,...,mj and E{Hj_k^2} = 1 for k =1,...,mj
                MatRGammaSj_iter_temp(1:mj,mj+1:mj+mj) = zeros(mj,mj);
                MatRGammaSj_iter_temp(mj+1:mj+mj,1:mj) = zeros(mj,mj);
                MatRGammaSj_iter  = MatRGammaSj_iter_temp;                       %    MatRGammaSj_iter(mhcj,mhcj)  
             end
             if ind_constraints == 3                                             %... constraints E{Hj] = 0 and E{Hj Hj'} = [I_mj]
                MatRGammaSj_iter_temp(1:mj,1+mj:mhcj) = zeros(mj,mhcj-mj);
                MatRGammaSj_iter_temp(1+mj:mhcj,1:mj) = zeros(mhcj-mj,mj);
                MatRGammaSj_iter_temp(1+mj:mj+mj,1+2*mj:mhcj) = zeros(mj,mhcj-2*mj);
                MatRGammaSj_iter_temp(1+2*mj:mhcj,1+mj:mj+mj) = zeros(mhcj-2*mj,mj);
                MatRGammaSj_iter  = MatRGammaSj_iter_temp;                         %    MatRGammaS_iter(mhc,mhc)  
             end 
             clear MatRGammaSj_iter_temp 
          end
          
          Rerrj(iter)          = norm(RGammaPj_iter)/norm(Rbcj);                   %  Rerrj(iter_limit,1)  
          RnormRlambdaj(iter)  = norm(Rlambdaj_iter_m1);                           %  RnormRlambdaj(iter_limit,1) 
          RcondGammaSj(iter)   = cond(MatRGammaSj_iter);                           %  RcondGammaSj(iter_limit,1)           
          disp('errj_iter ');disp(Rerrj(iter));
          disp('norm_lambdaj_iter ');disp(RnormRlambdaj(iter));
          disp('condGammaSj_iter ');disp(RcondGammaSj(iter));
          if iter >=2
             Rtolj(iter) = 2*abs(Rerrj(iter)-Rerrj(iter-1))/abs(Rerrj(iter)+Rerrj(iter-1));
             disp('tolj_iter '); disp(Rtolj(iter));
          end  
          fidlisting=fopen('listing.txt','a+'); 
          fprintf(fidlisting,'      \n '); 
          fprintf(fidlisting,'          ---iter number       = %7i \n ',iter);
          fprintf(fidlisting,'             errj_iter         = %14.7e \n ',Rerrj(iter));
          fprintf(fidlisting,'             tolj_iter         = %14.7e \n ',Rtolj(iter));
          fprintf(fidlisting,'      \n '); 
          fclose(fidlisting); 
          
         if Rerrj(iter) > 1e+3                                                      
             disp('----- STOP: Divergence of the iteration algorithm ')             
             disp(' Group number'); disp(j); 
             disp(' iter'); disp(iter);
             disp(' errj_iter'); disp(Rerrj(iter));
             disp('            Decrease alpha_relax2 or increase iter_relax2 ');
             fidlisting=fopen('listing.txt','a+');
             fprintf(fidlisting,'      \n ');
             fprintf(fidlisting,'      \n ');
             fprintf(fidlisting,'----- STOP: Divergence of the iteration algorithm  \n '); 
             fprintf(fidlisting,'      \n '); 
             fprintf(fidlisting,'      \n ');  
             fprintf(fidlisting,'       Group number     = %7i \n ',j); 
             fprintf(fidlisting,'       iter             = %7i \n ',iter);
             fprintf(fidlisting,'       errj_iter        = %14.7e \n ',Rerrj(iter));
             fprintf(fidlisting,'      \n ');
             fprintf(fidlisting,'       --- Decrease alpha_relax2 or increase iter_relax2\n ');
             fprintf(fidlisting,'      \n ');
             fclose(fidlisting); 
             error('STOP')                                                   %     STOP: divergence of ISDE
         end 
                                                                   %---convergence is reached, exit from the loop on iter  
         if (iter > 10) && (Rerrj(iter)-Rerrj(iter-1) > 0)                         %  a local minimum is obtained for iter - 1
             ind_convj       = 1; 
             MatReta_argj    = MatReta_argj_iter_m1;                                  %   MatReta_argj(mj,NnbMC), MatReta_argj_iter_m1(mj,NnbMC)
             iter_plot       = iter-1;
             disp('             Convergence with criterion 1')
             fidlisting=fopen('listing.txt','a+'); 
             fprintf(fidlisting,'      \n '); 
             fprintf(fidlisting,'             Convergence with criterion 1 \n ');
             fprintf(fidlisting,'      \n '); 
             fclose(fidlisting); 
             clear MatReta_argj_iter_m1 MatReta_argj_iter
             clear MatRhcj_iter Rmeanhcj_iter RGammaPj_iter Rlambdaj_iter_m1
             break                          % exit from the loop on iter  
          end                                                                             
         if ((minVarHj_iter >= minVarH) && (maxVarHj_iter <= maxVarH)) || Rerrj(iter) <= epsc % the variance of a component H_k is greater than 1 OR the error is less than the tolerance: exit
             ind_convj  = 1; 
             MatReta_argj    = MatReta_argj_iter;                                    %   MatReta_argj(mj,NnbMC), MatReta_argj_iter(mj,NnbMC)
             iter_plot       = iter;
             disp('             Convergence with criterion 2')
             fidlisting=fopen('listing.txt','a+'); 
             fprintf(fidlisting,'      \n '); 
             fprintf(fidlisting,'             Convergence with criterion 2 \n ');
             fprintf(fidlisting,'      \n '); 
             fclose(fidlisting); 
             clear MatReta_argj_iter_m1 MatReta_argj_iter
             clear  MatRhcj_iter Rmeanhcj_iter RGammaPj_iter Rlambdaj_iter_m1
             break                          % exit from the loop on iter
         end                                                                    
         if (iter > min(20,iter_limit) && Rerrj(iter) < 0.09 ) &&  (Rtolj(iter) < 0.01)     % the error is stationary: exit 
            ind_convj  = 1; 
            MatReta_argj = MatReta_argj_iter;                                       %   MatReta_argj(nu,NnbMC), MatReta_argj_iter(nu,NnbMC)
            iter_plot  = iter;
            disp('             Convergence with criterion 3')
            fidlisting=fopen('listing.txt','a+'); 
            fprintf(fidlisting,'      \n '); 
            fprintf(fidlisting,'             Convergence with criterion 3 \n ');
            fprintf(fidlisting,'      \n '); 
            fclose(fidlisting); 
            clear MatReta_argj_iter_m1 MatReta_argj_iter
            clear  MatRhcj_iter Rmeanhcj_iter RGammaPj_iter Rlambdaj_iter_m1
            break                         % exit from the loop on iter
         end
                                                                                    %---convergence not reached 
         if ((minVarHj_iter < minVarH) || (maxVarHj_iter > maxVarH)) && Rerrj(iter) > epsc
             iter_plot        = iter;
             Rtempj_iter      = MatRGammaSj_iter\RGammaPj_iter;                     %   Rtempj_iter = inv(MatRGammaSj_iter)*RGammaPj_iter             
             Rlambdaj_iter    = Rlambdaj_iter_m1 - Ralpha_relax(iter)*Rtempj_iter; %   Rlambdaj_iter(mhcj,1), Rlambdaj_iter_m1(mhcj,1)
             MatRlambdaj_iter =[];
             if ind_constraints == 3 && mj >= 2                                     %--- constraints E{Hj] = 0 and E{Hj Hj'} = [I_mj]
                MatRlambdaj_iter = zeros(mj,mj);                                    %  MatRlambda_iter(nu,nu) 
                ind = 0;
                for jj = 1:mj-1
                    for ii = jj+1:mj
                        ind = ind + 1;
                        MatRlambdaj_iter(jj,ii) = Rlambdaj_iter(2*mj+ind);
                        MatRlambdaj_iter(ii,jj) = MatRlambdaj_iter(jj,ii);
                    end
                end
             end             
             Rlambdaj_iter_m1     =  Rlambdaj_iter;
             MatRlambdaj_iter_m1  = MatRlambdaj_iter;  
             MatReta_argj_iter_m1 = MatReta_argj_iter;
             clear  MatRhcj_iter Rlambdaj_iter MatRlambdaj_iter Rmeanhcj_iter RGammaPj_iter MatRGammaSj_iter  Rtempj_iter
          end 
      end   %--- end for iter = 1:iter_limit  
      
      if ind_convj == 0               % iter_limit has been reached without convergence
         MatReta_argj = MatReta_argj_iter;                                       %   MatReta_argj(nu,NnbMC), MatReta_argj_iter(nu,NnbMC)
         iter_plot    = iter_limit;
         disp('------ No convergence of the iteration algorithm')
         disp('       iter_plot = '); disp(iter_plot) 
         disp('       Decrease alpha_relax2 or increase iter_relax2');   
         fidlisting=fopen('listing.txt','a+'); 
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'      \n ');
         fprintf(fidlisting,'          --- No convergence of the iteration algorithm    \n '); 
         fprintf(fidlisting,'                                                    \n ');
         fprintf(fidlisting,'              iter end          = %7i    \n ',iter_plot);
         fprintf(fidlisting,'              errj_iter         = %14.7e \n ',Rerrj(iter_plot));
         if iter >=2
            fprintf(fidlisting,'              tolj_iter         = %14.7e \n ',Rtolj(iter_plot));
         end
         fprintf(fidlisting,'              norm_lambdaj_iter = %14.7e \n ',RnormRlambdaj(iter_plot));
         fprintf(fidlisting,'              condGammaSj_iter  = %14.7e \n ',RcondGammaSj(iter_plot));
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'              --- Decrease alpha_relax2 or increase iter_relax2\n ');
         fprintf(fidlisting,'      \n '); 
         fclose(fidlisting); 
      end
      if ind_convj == 1 % convergence is reached
         disp('------ Convergence of the iteration algorithm')
         disp('       iter_plot = '); disp(iter_plot)   
         fidlisting=fopen('listing.txt','a+'); 
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'      \n ');
         fprintf(fidlisting,'          --- Convergence of the iteration algorithm    \n '); 
         fprintf(fidlisting,'                                                    \n ');
         fprintf(fidlisting,'              iter end          = %7i    \n ',iter_plot);
         fprintf(fidlisting,'              errj_iter         = %14.7e \n ',Rerrj(iter_plot));
         if iter >=2
            fprintf(fidlisting,'              toljj_iter         = %14.7e \n ',Rtolj(iter_plot));
         end
         fprintf(fidlisting,'              norm_lambdaj_iter = %14.7e \n ',RnormRlambdaj(iter_plot));
         fprintf(fidlisting,'              condGammaSj_iter  = %14.7e \n ',RcondGammaSj(iter_plot));
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'      \n '); 
         fclose(fidlisting); 
      end
                                                                %--- plot
      h = figure;      
      plot((1:1:iter_plot)',Rerrj(1:iter_plot),'b-')                                                 
      title(['Graph of function $\rm{err}_{',num2str(j),'}(\iota)$'],'FontSize',16,'Interpreter','latex','FontWeight','normal')                                         
      xlabel('$\iota$','FontSize',16,'Interpreter','latex')                                                                
      ylabel('$\rm{err}(\iota)$','FontSize',16,'Interpreter','latex')     
      saveas(h,['figure_STEP5_Rerr_',num2str(j),'.fig']); 
      close(h);	

      h = figure;      
      plot((1:1:iter_plot)',RnormRlambdaj(1:iter_plot),'b-')                                                 
      title(['Graph of function $\Vert \lambda_{',num2str(j),',\iota}\Vert $'],'FontSize',16,'Interpreter','latex','FontWeight','normal')                                         
      xlabel('$\iota$','FontSize',16,'Interpreter','latex')                                                                
      ylabel(['$\Vert \lambda_{',num2str(j),',\iota}\Vert $'],'FontSize',16,'Interpreter','latex')     
      saveas(h,['figure_STEP5_RnormRlambda_',num2str(j),'.fig']); 
      close(h);	
      
      h = figure;      
      plot((1:1:iter_plot)',RcondGammaSj(1:iter_plot),'b-')                                                 
      title('Graph of function $\rm{cond} [\Gamma"(\lambda_{\iota})]$','FontSize',16,'Interpreter','latex','FontWeight','normal')                                         
      xlabel('$\iota$','FontSize',16,'Interpreter','latex')                                                                
      ylabel(['$\rm{cond} [\Gamma"(\lambda_{',num2str(j),',\iota})]$'],'FontSize',16,'Interpreter','latex')     
      saveas(h,['figure_STEP5_RcondGammaS_',num2str(j),'.fig']); 
      close(h);                                       
      
      ind_print_argj = 1;
      if ind_print_argj == 1 && ind_convj == 1                  %--- The print of the mean and the covariance matrix of H_argj are bypassed if ind_print_argj = 0 
          RHmean_argj   = mean(MatReta_argj,2);
          MatRHcov_argj = cov(MatReta_argj'); 
          fidlisting=fopen('listing.txt','a+'); 
          fprintf(fidlisting,'      \n '); 
          fprintf(fidlisting,'      \n ');
          fprintf(fidlisting,'          --- RHmean_argj =          \n '); 
          fprintf(fidlisting,'          %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e \n ',RHmean_argj');
          fprintf(fidlisting,'      \n '); 
          fprintf(fidlisting,'      \n '); 
          fprintf(fidlisting,'          --- MatRHcov_argj =          \n '); 
          for k=1:mj       
              fprintf(fidlisting,'          %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e \n ',MatRHcov_argj(k,:));
              fprintf(fidlisting,'      \n ');
          end   
          fprintf(fidlisting,'      \n '); 
          fprintf(fidlisting,'      \n '); 
          fclose(fidlisting); 
          clear RHmean_argj MatRHcov_argj 
      end  
      return
end
   


 

 