
 function  [epsilonDIFFj,mDPj,mDMAPj,nbmDMAPj,Indgj,sj,shssj,shj,Deltarj,M0estimj,ximeanj,f0j] = ...
                                  sub_diffusion_maps_manifold_groupj_parameters(nu,N_d,jgroup,mj,MatReta_dj,ind_generator,icorrectif, ...
                                  Facteur,f0_ref,ind_f0,ind_DMAPS,ind_bandwidth_group,epsilonDIFF,mDP,mDMAP,nbmDMAP,Indg,epsilonDIFFmin_groups) 
%                                             
%-----------------------------------------------------------------------------------------------------------------------------------------
% Copyright C. Soize, 27 September 2020
%------------------------------------------------------------------------------------------------------------------------------------------
%
% estimation of parameters epsilonDIFFj, mDPj, mDMAPj, Indg(mDPj,1)
%--------------------------------------------------------------------------------------------------------------------------------------------
%
%--- INPUT  
%          nu            : dimension or H
%          N_d           : number of realizations in the initial dataset
%          mj            : length of vector Y^j = (H_jr1,... ,H_jrmj) of group j
%          MatReta_dj    : MatReta_dj(mj,N_d): realizations of Y^j of length mj
%          indgenerator, icorrectif, Facteur, f0_ref
%          ind_DMAPS            = 1 : use of mDMAP and MatRg(nu,mDMAP) constructed in STEP 3 (not mandatory, MatRg(nu,mDMAP) re-constructed in STEP5)
%                               = 2 : use of mDMAPj and MatRgj(mj,mDMAPj) constructed for each group j
%          ind_bandwidth_group  = 1 : use the bandwidth of Step 3 related to nu for each group j (recommended if ind_DMAPS = 1)
%                               = 2 : use the bandwidth related to mj of each group j (recommended if ind_DMAPS = 2)
%                     --- For ind_DMAPS = 1 use of the DMAPS parameters constructed in Step 3
%                     epsilonDIFF  :  eps for constructing the kernel in the diffusion maps
%                     mDP          :  Maximum number of dominant diffusion-map vectors computed (number of dominant eigenvalues)
%                     mDMAP        :  Dimension of the projection such that mDMAj <= mDP
%                     nbmDMAP      :  number of DMAP vectors retained for group j: nbmDMAP <= mDMAP <= mDP
%                     Indg         :  Indg(nbmDMAP ,1) contains the indices of the diffusion-map vectors used for the projection of the ISDE    
% 
%--- OUTPUT
%          epsilonDIFFj  :  eps for constructing the kernel in the diffusion maps
%          mDPj          :  Maximum number of dominant diffusion-map vectors computed (number of dominant eigenvalues)
%          mDMAPj        :  Dimension of the projection such that mDMAPj <= mDPj
%          nbmDMAPj      :  number of DMAP vectors retained for group j: nbmDMAPj <= mDMAPj <= mDPj
%          Indgj         :  Indgj(nbmDMAPj ,1) contains the indices of the diffusion-map vectors used for the projection of the ISDE    
%          sj,shssj,shj,Deltarj,M0estimj,ximeanj,f0j
%
%----------------------------------------------------------------------------------------------------------------------------------- 
                                                                         %--- calculating the parameters 
      if  ind_bandwidth_group ==  1                                                              
          s = ((4/((nu+2)*N_d))^(1/(nu+4)));                             % Silver bandwidth          
          modifs = 1;                                                    % in the usual theory, modifs=1;          
          s = modifs*s;                                                  % Silver bandwidth modified 
          s2 = s*s;
          if icorrectif == 0 || icorrectif == 1
             if icorrectif == 0
                shss =1;
             end
             if icorrectif == 1
                shss = 1/sqrt(s2+(N_d-1)/N_d);                           % coef sh/s
             end
          else
              error('STOP: icorrectif must be equal to 0 or equal to 1')
          end      
          sh = s*shss;                                                                    
          sj    = s;
          shssj = shss;
          shj   = sh;
      end
      
      if  ind_bandwidth_group ==  2  
          sj = ((4/((mj+2)*N_d))^(1/(mj+4)));                             % Silver bandwidth          
          modifs = 1;                                                     % in the usual theory, modifs=1;          
          sj = modifs*sj;                                                  % Silver bandwidth modified 
          sj2 = sj*sj;
          if icorrectif == 0 || icorrectif == 1
             if icorrectif == 0
                shssj =1;
             end
             if icorrectif == 1
                shssj = 1/sqrt(sj2+(N_d-1)/N_d);                           % coef sh/s
             end
          else
              error('STOP: icorrectif must be equal to 0 or equal to 1')
          end       
          shj = sj*shssj;   
      end
      
      if ind_f0 == 0
         f0j = f0_ref;        
      end
      if ind_f0 == 1
         f0j = f0_ref/shj;         % this the maximum value for f0j
      end
      
      Deltarj = 2*pi*shj/Facteur;  
      ximeanj = f0j*shj/4;  
                                                                     % estimation  M0estim of the number of sampling points in the transient solution. 
                                                                     % Duration R = M0estim*jDeltarj for an attenuation of 1/100
      M0estimj = 2*log(100)*Facteur/(pi*f0j*shj);                    % reference value for a MONTECARLO generation instead of an ERGODIC generation

  %-------------------------------------------------------------------------------------------------------------------------------------------------
  %        case  ind_generator = 0
  %-------------------------------------------------------------------------------------------------------------------------------------------------
  
     if ind_generator == 0
        mDMAPj       = N_d;                    % maximum value of mDMAP for group j  
        nbmDMAPj     = N_d;                    % number of DMAP vector retained for group j
        epsilonDIFFj = 0;
        mDPj         = 0;
        Indgj        = 0;
     end
  %-------------------------------------------------------------------------------------------------------------------------------------------------
  %        case  ind_generator = 1 and ind_DMAPS = 1 : use of mDMAP and MatRg(nu,mDMAP) constructed in STEP 3 (not mandatory
  %-------------------------------------------------------------------------------------------------------------------------------------------------   
  
     if ind_generator == 1 && ind_DMAPS == 1 
        mDMAPj       = mDMAP;
        nbmDMAPj     = nbmDMAP;
        epsilonDIFFj = epsilonDIFF;
        mDPj         = mDP;
        Indgj        = Indg;
     end                                        %--- end if ind_generator = 1 AND ind_DMAPS = 1
     
   %-------------------------------------------------------------------------------------------------------------------------------------------------
   %        case  ind_generator = 1  AND  ind_DMAPS = 2  AND   mj = 1 : MatRg(N_d,1) = MatReta_d(1,N_d)'
   %-------------------------------------------------------------------------------------------------------------------------------------------------   
  
     if (ind_generator == 1 && ind_DMAPS == 2) && (mj == 1) 
        mDMAPj       = 1;
        nbmDMAPj     = 1;
        epsilonDIFFj = 0;
        mDPj         = 1;
        Indgj        = 1;
     end                                        %--- end if ind_generator = 1 AND ind_DMAPS = 2   
  %-------------------------------------------------------------------------------------------------------------------------------------------------
  %        case  ind_generator = 1  AND  ind_DMAPS = 2  AND  mj >=2 : use of mDMAPj and MatRgj(mj,mDMAPj) constructed for each group j
  %-------------------------------------------------------------------------------------------------------------------------------------------------   
  
     if (ind_generator == 1 && ind_DMAPS == 2) && (mj >=2)
      
        iterlimit    = 500;
        
        mDMAPj       = mj + 1;                  % maximum value of mDMAP for group j 
        mDPj         = min(mDMAPj + 10, N_d);   
        nbmDMAPj     = mDMAPj - 1;              % number of DMAP vector retained for group j 
        Indgj        = (2:1:mDMAPj)';           % indices of the DMAP vectors retained for group j 
%         nbmDMAPj     = mDMAPj ;                % number of DMAP vector retained for group j 
%         Indgj        = (1:1:mDMAPj)';          % indices of the DMAP vectors retained for group j 
         
        if mDMAPj <= 4          
           epsilonDIFFj_iterm1 = 0.5;                              % lower bound
           step_epsilonDIFFj   = 0.1;
        end
        if mDMAPj >=5 && mDMAPj  <= 10          
           epsilonDIFFj_iterm1 = 1;                                 % lower bound
           step_epsilonDIFFj   = 0.1;
        end
        if mDMAPj > 10          
           epsilonDIFFj_iterm1 = epsilonDIFFmin_groups;             % lower bound
           step_epsilonDIFFj   = 1 + fix(epsilonDIFFmin_groups/50);
        end
        
        Rcomp_groupj = zeros(iterlimit,1);
        
        for iter = 1:iterlimit
         
            epsilonDIFFj_iter = epsilonDIFFj_iterm1 + step_epsilonDIFFj;
            
           [MatRKernel] = sub_Kernel1(N_d,mj,MatReta_dj,epsilonDIFFj_iter);
            scaling     = max(max(MatRKernel));                                       %--- removing the negative zeros of the eigenvalues of MatRKernel
            MatRKernel  = MatRKernel + scaling*1e-12*eye(N_d,N_d);
                                                                         %--- compute the transition probability matrix MatRP and its symmetrized matrix MatRPs
            MatRPs = zeros(N_d,N_d);                                     %    MatRPs(N_d,N_d)
            Rd   = sum(MatRKernel,2);
            Rdm1s2 = 1./sqrt(Rd);
            for i=1:N_d
                MatRPs(i,:) = Rdm1s2(i)*MatRKernel(i,:).*Rdm1s2';         %    MatRPs(N_d,N_d), Symmetric matrix
            end
            clear MatRKernel scaling Rd  Rdm1s2
            MatRPs = 0.5*(MatRPs+MatRPs');                                %    MatRPs(N_d,N_d), imposing a perfect symmetry
                                       
                                                                          %--- compute the eigenvalues of  MatRPs * MatRphi = MatRphi * diag(Rlambda) 
           if 2*mDPj+8 >= N_d                                             % computation with eig
              [~,MatRdiag] =  eig(MatRPs);                                % MatRPs(N_d,N_d)
              Rlambda1   = diag(MatRdiag);                                % Rlambda1(N_d,1)
              [~,Index]  = sort(1./abs(Rlambda1));                        % modulus in descending order
              Rlambda    = Rlambda1(Index);                               % Rlambda(N_d,1)   
              if mDPj + 1 <= N_d
                 Rlambda(mDPj+1:N_d)   = [];                              % Rlambda(mDPj,1)  
              end
           end
           if 2*mDPj+8 < N_d                                              % computation with eigs
              OPTS.tol   = 1e-10;                                         % convergence tolerance
              OPTS.p     = 2*mDPj+8;                                      % dimension of the Arnoldi basis
              OPTS.maxit = 300;                                           % maximum number of iterations
              OPTS.disp  = 0;                                             % number of eigenvalues displayed at each iteration
              [~,MatRdiag] =  eigs(MatRPs,mDPj,'LM',OPTS);          
              Rlambda1 = diag(MatRdiag);                                  % Rlambda1(mDPj,1)
             [~,Index]  = sort(1./abs(Rlambda1));                         % modulus in descending order
              Rlambda = Rlambda1(Index);                                  % Rlambda(mDPj,1) 
           end
           clear  MatRdiag MatRPs Rlamnda1 Index 
                                                                          %--- checking the jump in the eigenvalues spectrum
           if mDMAPj+1 <= mDPj
              comp  = Rlambda(mDMAPj+1)/Rlambda(2); 
              compb = Rlambda(mDMAPj+1)/Rlambda(mDMAPj); 
           end
           if mDMAPj+1 >  mDPj
              comp = Rlambda(mDMAPj)/Rlambda(2); 
              compb = comp;
           end  
           
           Rcomp_groupj(iter) = comp;
           
           if mDMAPj <= 10 && (comp <= 0.1  && compb <=0.1)               % solution is epsilonDIFFj_iter
              epsilonDIFFj  =  epsilonDIFFj_iter;
              break
           end
           if mDMAPj > 10 && comp <= 0.1                                  % solution is epsilonDIFFj_iter
              epsilonDIFFj  =  epsilonDIFFj_iter;
              break
           end     
           if iter == iterlimit
              if mDMAPj+1 <= mDPj
                 if mDMAPj <= 10
                    h = figure; 
                    plot(Rcomp_groupj,'b-o')
                    title(['Rcomp group',num2str(jgroup),'(iter)'],'FontWeight','Normal','FontSize',16,'Interpreter','latex')
                    xlabel('iter','FontSize',16,'Interpreter','latex') 
                    ylabel(['Rcomp group',num2str(jgroup),'(iter)'],'FontSize',16,'Interpreter','latex') 
                    saveas(h,['figure_STEP5_Rcomp group',num2str(jgroup),'.fig']);  
                    close(h); 
                    fidlisting=fopen('listing.txt','a+'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'STOP: iterlimit reached without finding a solution mj = %7i \n ',jgroup); 
                    fclose(fidlisting);  
                    error('STOP: iterlimit reached without finding a solution')
                 end
                 if mDMAPj > 10
                    h = figure; 
                    plot(Rcomp_groupj,'b-o')
                    title(['Rcomp group',num2str(jgroup),'(iter)'],'FontWeight','Normal','FontSize',16,'Interpreter','latex')
                    xlabel('iter','FontSize',16,'Interpreter','latex') 
                    ylabel(['Rcomp group',num2str(jgroup),'(iter)'],'FontSize',16,'Interpreter','latex') 
                    saveas(h,['figure_STEP5_Rcomp group',num2str(jgroup),'.fig']);  
                    close(h); 
                    fidlisting=fopen('listing.txt','a+'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'STOP: iterlimit reached without finding a solution mj = %7i \n ',jgroup); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'      Try to increase the value of epsilonDIFFmin_groups'); 
                    fclose(fidlisting);  
                    error('STOP: iterlimit reached without finding a solution')
                 end
                 disp('      mj = '); disp(jgroup)
                 error('STOP: iterlimit reached without finding a solution')
              end
              if mDMAPj+1 > mDPj
                 h = figure; 
                 plot(Rcomp_groupj,'b-o')
                 title(['Rcomp group',num2str(jgroup),'(iter)'],'FontWeight','Normal','FontSize',16,'Interpreter','latex')
                 xlabel('iter','FontSize',16,'Interpreter','latex') 
                 ylabel(['Rcomp group',num2str(jgroup),'(iter)'],'FontSize',16,'Interpreter','latex') 
                 saveas(h,['figure_STEP5_Rcomp group',num2str(jgroup),'.fig']);  
                 close(h); 
                 fidlisting=fopen('listing.txt','a+'); 
                 fprintf(fidlisting,'      \n '); 
                 fprintf(fidlisting,'STOP: iterlimit reached without finding a solution \n'); 
                 fprintf(fidlisting,'      and mDMAP = mDP: diffusionmaps cannot be used \n'); 
                 fprintf(fidlisting,'      re-run with ind_generator = 0. mj = %7i \n ',jgroup); 
                 fclose(fidlisting);  
                 disp('STOP: iterlimit reached without finding a solution')
                 disp('      and mDMAP = mDP: diffusionmaps cannot be used')
                 disp('      re-run with ind_generator = 0')
                 disp('      mj = '); disp(jgroup)
                 error('STOP: iterlimit reached without finding a solution') 
              end
           end
           epsilonDIFFj_iterm1 = epsilonDIFFj_iter;
        end                                         % end loop iter  
        
        h = figure; 
        plot(Rcomp_groupj,'b-o')
        title(['Rcomp group',num2str(jgroup),'(iter)'],'FontWeight','Normal','FontSize',16,'Interpreter','latex')
        xlabel('iter','FontSize',16,'Interpreter','latex') 
        ylabel(['Rcomp group',num2str(jgroup),'(iter)'],'FontSize',16,'Interpreter','latex') 
        saveas(h,['figure_STEP5_Rcomp group',num2str(jgroup),'.fig']);  
        close(h);     
        
     end                                        %--- end if ind_generator = 1 AND ind_DMAPS = 2
      
     return
end
   


 

 