
 function  [epsilonDIFF,mDP,mDMAP,nbmDMAP,Indg,MatRg,Rlambda] = sub_diffusion_maps_parameters_step5(nu,N_d,MatReta_d,ind_generator,tdiffusion,epsilonDIFFmin) 
%                                             
%-----------------------------------------------------------------------------------------------------------------------------------------
% Copyright C. Soize, 19 November 2020
%------------------------------------------------------------------------------------------------------------------------------------------
%
% estimation of parameters epsilonDIFF, mDP, mDMAP, Indg(nbmDMAP,1), and computing MatRg, Rlambda
%--------------------------------------------------------------------------------------------------------------------------------------------
%
%--- INPUT  
%          nu            : dimension or H
%          N_d           : number of realizations in the initial dataset
%          MatReta_d     : MatReta_d(nu,N_d) 
%          indgenerator
%          tdiffusion
%          epsilonDIFFmin
%
%--- OUTPUT
%          epsilonDIFF   :  eps for constructing the kernel in the diffusion maps
%          mDP           :  Maximum number of dominant diffusion-map vectors computed (number of dominant eigenvalues)
%          mDMAP         :  Dimension of the projection such that mDMAP <= mDP
%          nbmDMAP       :  number of DMAP vectors : nbmDMAP <= mDMAP <= mDP
%          Indg          :  Indg(nbmDMAP ,1) contains the indices of the diffusion-map vectors used for the projection of the ISDE 
%          MatRg(N_d,nbmDMAP)
%          Rlambda(mDP)
%
%----------------------------------------------------------------------------------------------------------------------------------- 

     if ind_generator == 0
        mDMAP       = N_d;                    % maximum value of mDMAP 
        nbmDMAP     = N_d;                    % number of DMAP vector 
        epsilonDIFF = 0;
        mDP         = 0;
        Indg        = 0;
        MatRg       = eye(N_d,N_d);
        Rlambda     = eye(N_d,1);
     end
     
     if ind_generator == 1
      
        %----- Iteration algorithm for computing epsilonDIFF
        
        iterlimit    = 500;
        
        mDMAP        = nu + 1;                %  maximum value of the numbers of DMAPS basis  
        mDP          = min(mDMAP + 20, N_d);   
        nbmDMAP      = mDMAP - 1;             % number of DMAP vector retained 
        Indg         = (2:1:mDMAP)';          % indices of the DMAP vectors retained 
%         nbmDMAP     = mDMAP ;               % number of DMAP vector retained 
%         Indg        = (1:1:mDMAP)';         % indices of the DMAP vectors retained 
         
        if mDMAP <= 4          
           epsilonDIFF_iterm1 = 0.5;          % lower bound
           step_epsilonDIFF   = 0.1;
        end
        if mDMAP >=5 && mDMAP <= 10          
           epsilonDIFF_iterm1 = 1;            % lower bound
           step_epsilonDIFF   = 0.1;
        end
        if mDMAP > 10 && mDMAP <= 20          
           epsilonDIFF_iterm1 = 1;                          % lower bound
           step_epsilonDIFF  = 1;
        end
        if mDMAP > 20          
           epsilonDIFF_iterm1 = epsilonDIFFmin;             % lower bound
           step_epsilonDIFF   = 1 + fix(epsilonDIFFmin/50);
        end
        
        Rcomp = zeros(iterlimit,1);
        
        for iter = 1:iterlimit
         
            epsilonDIFF_iter = epsilonDIFF_iterm1 + step_epsilonDIFF;
            
           [MatRKernel] = sub_Kernel1(N_d,nu,MatReta_d,epsilonDIFF_iter);
            scaling     = max(max(MatRKernel));                                       %--- removing the negative zeros of the eigenvalues of MatRKernel
            MatRKernel  = MatRKernel + scaling*1e-12*eye(N_d,N_d);
                                                                         %--- compute the transition probability matrix MatRP and its symmetrized matrix MatRPs
            MatRPs = zeros(N_d,N_d);                                     %    MatRPs(N_d,N_d)
            Rd   = sum(MatRKernel,2);
            Rdm1s2 = 1./sqrt(Rd);
            for i=1:N_d
                MatRPs(i,:) = Rdm1s2(i)*MatRKernel(i,:).*Rdm1s2';         %    MatRPs(N_d,N_d), Symmetric matrix
            end
            clear MatRKernel scaling Rd  Rdm1s2
            MatRPs = 0.5*(MatRPs+MatRPs');                                %    MatRPs(N_d,N_d), imposing a perfect symmetry
                                       
                                                                          %--- compute the eigenvalues of  MatRPs * MatRphi = MatRphi * diag(Rlambda) 
           if 2*mDP+8 >= N_d                                              % computation with eig
              [~,MatRdiag] =  eig(MatRPs);                                % MatRPs(N_d,N_d)
              Rlambda1   = diag(MatRdiag);                                % Rlambda1(N_d,1)
              [~,Index]  = sort(1./abs(Rlambda1));                        % modulus in descending order
              Rlambda    = Rlambda1(Index);                               % Rlambda(N_d,1)   
              if mDP + 1 <= N_d
                 Rlambda(mDP+1:N_d)   = [];                               % Rlambda(mDP,1)  
              end
           end
           if 2*mDP+8 < N_d                                               % computation with eigs
              OPTS.tol   = 1e-10;                                         % convergence tolerance
              OPTS.p     = 2*mDP+8;                                       % dimension of the Arnoldi basis
              OPTS.maxit = 300;                                           % maximum number of iterations
              OPTS.disp  = 0;                                             % number of eigenvalues displayed at each iteration
              [~,MatRdiag] =  eigs(MatRPs,mDP,'LM',OPTS);          
              Rlambda1 = diag(MatRdiag);                                  % Rlambda1(mDPj,1)
             [~,Index]  = sort(1./abs(Rlambda1));                         % modulus in descending order
              Rlambda = Rlambda1(Index);                                  % Rlambda(mDP,1) 
           end
           clear  MatRdiag MatRPs Rlamnda1 Index 
                                                                           %--- checking the jump in the eigenvalues spectrum
           if mDMAP+1 <= mDP
              comp  = Rlambda(mDMAP+1)/Rlambda(2); 
              compb = Rlambda(mDMAP+1)/Rlambda(mDMAP);
           end
           if mDMAP+1 >  mDP
              comp = Rlambda(mDMAP)/Rlambda(2); 
              compb = comp;
           end
           
            Rcomp(iter) = comp;           
           
           if mDMAP <= 20 && (comp <= 0.1  && compb <=0.1)               % solution is epsilonDIFF_iter
              epsilonDIFF  =  epsilonDIFF_iter;
              break
           end
           if mDMAP > 20 && comp <= 0.1                                  % solution is epsilonDIFF_iter
              epsilonDIFF  =  epsilonDIFF_iter;
              break
           end 
            
           if iter == iterlimit
              if mDMAP+1 <= mDP
                 if mDMAP <= 20
                    h = figure; 
                    plot(Rcomp,'b-o')
                    title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
                    xlabel('iter','FontSize',16,'Interpreter','latex') 
                    ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
                    saveas(h,'figure_STEP5_Rcomp.fig');  
                    close(h);
                    fidlisting=fopen('listing.txt','a+'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'STOP: iterlimit reached without finding a solution'); 
                    fclose(fidlisting);  
                    error('STOP: iterlimit reached without finding a solution')
                 end
                 if mDMAP > 20
                    h = figure; 
                    plot(Rcomp,'b-o')
                    title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
                    xlabel('iter','FontSize',16,'Interpreter','latex') 
                    ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
                    saveas(h,'figure_STEP5_Rcomp.fig');  
                    close(h);
                    fidlisting=fopen('listing.txt','a+'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'STOP: iterlimit reached without finding a solution'); 
                    fprintf(fidlisting,'      \n '); 
                    fprintf(fidlisting,'      Try to increase the value of epsilonDIFFmin'); 
                    fclose(fidlisting);  
                    error('STOP: iterlimit reached without finding a solution')
                 end
              end
              if mDMAP+1 > mDP
                 h = figure; 
                 plot(Rcomp,'b-o')
                 title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
                 xlabel('iter','FontSize',16,'Interpreter','latex') 
                 ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
                 saveas(h,'figure_STEP5_Rcomp.fig');
                 close(h);
                 fidlisting=fopen('listing.txt','a+'); 
                 fprintf(fidlisting,'      \n '); 
                 fprintf(fidlisting,'STOP: iterlimit reached without finding a solution \n'); 
                 fprintf(fidlisting,'      and mDMAP = mDP: diffusionmaps cannot be used \n'); 
                 fprintf(fidlisting,'      re-run with ind_generator = 0'); 
                 fclose(fidlisting);  
                 disp('STOP: iterlimit reached without finding a solution')
                 disp('      and mDMAP = mDP: diffusionmaps cannot be used')
                 disp('      re-run with ind_generator = 0')
                 error('STOP: iterlimit reached without finding a solution') 
              end
           end
           epsilonDIFF_iterm1 = epsilonDIFF_iter;
        end                                        %--- end loop iter 
        
        h = figure; 
        plot(Rcomp,'b-o')
        title('Rcomp(iter)','FontWeight','Normal','FontSize',16,'Interpreter','latex')
        xlabel('iter','FontSize',16,'Interpreter','latex') 
        ylabel('Rcomp(iter)','FontSize',16,'Interpreter','latex') 
        saveas(h,'figure_STEP5_Rcomp.fig');  
        close(h);
        
        %----- computing MatRg(nu,nbmDMAP) and MatRlambda(mDP,1)
        
        [MatRKernel] = sub_Kernel1(N_d,nu,MatReta_d,epsilonDIFF);
                                                                     %    removing the negative zeros of the eigenvalues of MatRKernel
         scaling = max(max(MatRKernel));
         MatRKernel = MatRKernel + scaling*1e-12*eye(N_d,N_d);
                                                                     %    compute the transition probability matrix MatRP and its symmetrized matrix MatRPs
         MatRPs = zeros(N_d,N_d);                                    %    MatRPs(N_d,N_d)
         Rd   = sum(MatRKernel,2);
         Rdm1s2 = 1./sqrt(Rd);
         for i=1:N_d
             MatRPs(i,:) = Rdm1s2(i)*MatRKernel(i,:).*Rdm1s2';       %    MatRPs(N_d,N_d), Symmetric matrix
         end
         clear MatRKernel Rd
         MatRPs = 0.5*(MatRPs+MatRPs');                              %    MatRPs(N_d,N_d), imposing a perfect symmetry
                                           
                                                                     %    compute MatRphi : MatRPs * MatRphi = MatRphi * diag(Rlambda) 

         if 2*mDP+8 >= N_d                                           % computation with eig
            [MatRphi1,MatRdiag] =  eig(MatRPs);                      % MatRPs(N_d,N_d)
            Rlambda1   = diag(MatRdiag);                             % Rlambda1(N_d,1)
            [~,Index]  = sort(1./abs(Rlambda1));                     % modulus in descending order
            Rlambda    = Rlambda1(Index);                            % Rlambda(N_d,1)                          
            MatRphi    = MatRphi1(:,Index);                          % MatRphi(N_d,N_d)
            if mDP + 1 <= N_d
               Rlambda(mDP+1:N_d)   = [];                            % Rlambda(mDP,1)                          
               MatRphi(:,mDP+1:N_d) = [];                            % MatRphi(N_d,mDP)
            end
         end

         if 2*mDP+8 < N_d                                           % computation with eigs
            OPTS.tol   = 1e-10;                                     % convergence tolerance
            OPTS.p     = 2*mDP+8;                                   % dimension of the Arnoldi basis
            OPTS.maxit = 300;                                       % maximum number of iterations
            OPTS.disp  = 0;                                         % number of eigenvalues displayed at each iteration
            [MatRphi1,MatRdiag] =  eigs(MatRPs,mDP,'LM',OPTS);      % MatRphi1(N_d,mDP)
            Rlambda1 = diag(MatRdiag);                              % Rlambda1(mDP,1)
           [~,Index]  = sort(1./abs(Rlambda1));                     % modulus in descending order
            Rlambda = Rlambda1(Index);                              % Rlambda(mDP,1)                          
            MatRphi = MatRphi1(:,Index);                            % MatRphi(N_d,mDP)
         end
         clear Rlambda1 Rbid Index  MatRPhi1 MatRdiag
         clear MatRPs 


         fidlisting=fopen('listing.txt','a+'); 
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'      \n ');                     
         fprintf(fidlisting,'--- PARAMETERS OF THE DMAPS BASIS COMPUTED IN STEP 5 \n ');   
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'      \n ');  
         fprintf(fidlisting,'epsilonDIFF = %14.7e \n ',epsilonDIFF); 
         fprintf(fidlisting,'tdiffusion  = %8.4f \n ',tdiffusion);
         fprintf(fidlisting,'mDP         = %7i \n ',mDP); 
         fprintf(fidlisting,'mDMAP       = %7i \n ',mDMAP); 
         fprintf(fidlisting,'nbmDMAP     = %7i \n ',nbmDMAP); 
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'      \n ');  
         fprintf(fidlisting,'Indg =          \n '); 
         fprintf(fidlisting,' %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i %3i \n ',Indg');
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'      \n '); 
         fprintf(fidlisting,'--- EIGENVALUES OF THE TRANSITION MATRIX COMPUTED IN STEP 5 \n ');                        
         fprintf(fidlisting,'               \n '); 
         Rnum=(1:1:mDP)'; 
         MatRprint = [Rnum Rlambda(1:1:mDP)]';                                        
         fprintf(fidlisting,'%5i %18.11e  \n',MatRprint);  
         fclose(fidlisting); 
                                                                    % Plot
         hold off 
         h=figure;                                
         semilogy((1:1:mDP)',Rlambda(1:1:mDP),'-o') 
         title('Eigenvalues $\Lambda_\alpha$ of the transition matrix in $\log_{10}$ scale computed in Step5','FontSize',16,'Interpreter','latex')                                         
         xlabel('$\alpha$','FontSize',16,'Interpreter','latex')                                                                
         ylabel('$\Lambda_\alpha$','FontSize',16,'Interpreter','latex')
         saveas(h,'figure_STEP5_eigenvalueDMAPS.fig'); 
         hold off
         close(h)
                                                                    % compute MatRPsi : MatRP * MatRPsi = MatRPsi * diag(Rlambda)
         MatRPsi = zeros(N_d,mDP);                            
         for k=1:mDP
             MatRPsi(:,k) = MatRphi(:,k).*Rdm1s2;        
         end  
         Rlambdat = Rlambda.^tdiffusion; 
         clear MatRphi Rdm1s2
                                                                    % loading vectors of the diffusion maps
         MatRg = zeros(N_d,nbmDMAP);                                % MatRg(N_d,nbmDMAPj) selecting the nbmDMAPj diffusion maps with Indgj(1),...,Indgj(nbmDMAPj)      
         for beta=1:nbmDMAP                                       
             MatRg(:,beta) = Rlambdat(Indg(beta))*MatRPsi(:,Indg(beta));              % MatRg(N_d,nbmDMAP) 
         end
         clear Rlambdat MatRPsi                      
     end                                          %--- end if ind_generator == 1
     return
end
   


 

 