function [MatRLU] = sub_gener_3D_ireduc1(Npoint,RX1,RX2,RX3,L1,L2,L3)  %
                                                                  %
%----------------------------------------------------------------------------------------------------------------------
% C. Soize mars 2004 
%
% Gnration de la matrice [LU] telle que [CU] =  [LU]^T [LU] avec [CU]_{ij} = RU(x^i - x^j) o
% RU(eta) est la fonction d'autocorrelation du champ x =(x1,x2,x3) ---> U(x) index sur R^3, gaussien,
% centr, tels que E{U}(x)^2}=1, homomogne sur R^3, tel que
% RU(eta) = RU(eta1,eta2,eta3)= rho_1(eta1) rho_2(eta2) rho_3(eta3)     avec
% rho1(eta1) = {2 L1 / pi eta1 ) sin(pi eta1 / 2 L1) }^2
% rho2(eta2) = {2 L2 / pi eta2 ) sin(pi eta2 / 2 L2) }^2
% rho3(eta3) = {2 L3 / pi eta3 ) sin(pi eta3 / 2 L3) }^2
%
% le paramtre L1 reprsente la longueur de corrlation en x1 du champ x =(x1,x2,x3) ---> U(x).
% le paramtre L2 reprsente la longueur de corrlation en x2 du champ x =(x1,x2,x3) ---> U(x).
% le paramtre L3 reprsente la longueur de corrlation en x3 du champ x =(x1,x2,x3) ---> U(x).
%
% Cette subroutine doit etre excute en dehors de la boucle des tirages (en initialisation).
%
%--- INPUT parameters
%
%         Npoint     = nombre de points x^1,...,x^N de R^3
%         RX1        = vecteur de la coordonne x1 des points x^1,...,x^N
%         RX2        = vecteur de la coordonne x2 des points x^1,...,x^N
%         RX3        = vecteur de la coordonne x3 des points x^1,...,x^N
%         L1         = longueur de corrlation dans la direction x1
%         L2         = longueur de corrlation dans la direction x2
%         L3         = longueur de corrlation dans la direction x3
%
%--- OUTPUT parameter : (N,N) upper triangular matrix LU  (ne pas mettre en sparse, calcul plus longs)
%
%-----------------------------------------------------------------------------------------------------------------------
                                                      %
MatRCU = zeros(Npoint,Npoint);                        %---initialisation
                                                      %
parfor j = 1:Npoint                                      %---Construction de la matrice CU
    Reta1 = RX1 - RX1(j);                             %
    Reta2 = RX2 - RX2(j);                             %
    Reta3 = RX3 - RX3(j);                             %
    Rbid1 = pi*Reta1/(2*L1) + 1.e-6;                  %
    Rbid2 = pi*Reta2/(2*L2) + 1.e-6;                  %
    Rbid3 = pi*Reta3/(2*L3) + 1.e-6;                  %
    MatRCU(:,j) = ( (sin(Rbid1)./Rbid1).^2 ).* ...    %
                  ( (sin(Rbid2)./Rbid2).^2 ).* ...    %
                  ( (sin(Rbid3)./Rbid3).^2 );         %
end                                                   %
                                                      %
for j = 1:Npoint                                      %
    MatRCU(j,j) = 1;                                  %
end                                                   %
                                                      %---Symtrisation
MatRCU =  0.5*(MatRCU+MatRCU');                       %
                                                      %---Construction de LU: [CU] =  [LU]^T [LU]
MatRLU = chol(MatRCU);                                %
                                                      %
clear Reta1 Reta2 Reta3 Rbid1 Rbid2 Rbid3 MatRCU      %
return                                                %
