function [MatRW] = sub_gener_MatRgr1(m,nr,RUU)

%    GENERATION of nr independent realizations of a random vector W = (W_1, ... , W_m) of dimension m,
%               with DEPENDENT random components W_1, ... , W_m
%               generated from the monomials with degrees from 1 to m, of a unique random variable Xi (uniform or Gaussian)
%                                              M_k = Xi^k               with k = 1, ... , m (max degree is Nd = m)
%               Z = (Z_1, ... , Z_m)     with  Z_i = sum_{k=1}^i M_k    and with i = 1,...,m 
%               W = (W_1, ... , W_m)     is normalized in order that E{W} = 0  an,d E{W W'] = [I_m]
%
%--- INPUT : 
%     m       = length of random vector W
%     Ndegree = m is the maximum degree of the univariate monomial
%     nr      = number of independent realizations
%
%--- OUTPUT
%    MatRW(m,nr)

  Ndegree = m;  
  RXi     = RUU(1,:);                        % nr independent realizations for the germ Xi 
  MatRM =zeros(Ndegree,nr);                  % nr independent realizations of the monomials M_k = Xi^k with k = 1, ... , Ndegree
  MatRM(1,:) = RXi;
  if Ndegree >= 2
     for k = 2:Ndegree 
         RPk = sqrt(k)*RXi;
         MatRM(k,:) = MatRM(k-1,:).*RPk;   % MatRM(k,ell) = sqrt(k!) M_k^ell
     end
  end
                                           %--- Construction of a random vector Z with m components such that
                                           %    Z_i = sum_{k=0}^i sqrt(k!) M_k with i = 1,...,m
  MatRZ = zeros(m,nr);                     %    MatRZ(i,ell) = Z_i^ell , ell = 1, ... , nr  and i = 1,...,m
  MatRZ(1,:) = ones(1,nr)+ MatRM(1,:);   
  if m >= 2
     for i = 2:m
         clear RSi
         RSi = ones(1,nr);
         for k=1:i
             RSi = RSi + MatRM(k,:);
         end
         MatRZ(i,:) = RSi;
     end
  end    
                                            %--- Normalisation
  RmeanZ =mean(MatRZ,2);
  MatRZ0 = MatRZ - repmat(RmeanZ,1,nr);
  MatRcovZ0 = cov(MatRZ0');
  [MatRL] = chol(MatRcovZ0);
  MatRW = (MatRL')\MatRZ0;
%   RmeanW   = mean(MatRW,2);
%   MatRcovW = cov(MatRW');
  
return                                                                        
