function [MatRetagr] = sub_gener_MatRgr2(m,nr,MatRUU)

%    GENERATION of nr independent realizations of a random vector Hgr = (Hgr_1, ... , Hgr_m) of dimension m,
%               with DEPENDENT random components Hgr_1, ... , Hgr_m
%               generated from the monomials with degrees from 1 to m, of a unique random variable Xi (uniform or Gaussian)
%                                              M_k = Xi^k               with k = 1, ... , m (max degree is m)
%               Z   = (Z_1, ... , Z_m)         with  Z_i = sum_{k=1}^i M_k    and with i = 1,...,m 
%               Hgr = (Hgr_1, ... , Hgr_m)     is normalized in order that E{Hgr} = 0  and E{Hgr Hgr'] = [I_m]
%
%--- INPUT : 
%     m       = length of random vector Hgr
%     nr      = number of independent realizations
%     MatRUU(m,nr)
%
%--- OUTPUT
%    MatRetagr(m,nr)
  
  MatRM      = zeros(m,nr);                  % nr independent realizations of the monomials M_k = UU_k^k with k = 1, ... , m
  MatRM(1,:) = MatRUU(1,:);
  if m >= 2
     coefj = 1;
     for j = 2:m 
         coefj = coefj*sqrt(j);
         degreej = j;
         MatRM(j,:) = coefj*MatRUU(j,:).^degreej;   % MatRM(j,ell) = sqrt(j!) M_j^j
     end
  end
%                                            %--- Construction of a random vector Z with m components such that
%                                            %    Z_j = sum_{k=0}^j sqrt(k!) M_k with j = 1,...,m
%   MatRZ = zeros(m,nr);                     %    MatRZ(j,ell) = Z_j^ell , ell = 1, ... , nr  and j = 1,...,m
%   MatRZ(1,:) = ones(1,nr)+ MatRM(1,:).^2;   % MODIFICATIONS-MODIFICATIONS
%   if m >= 2
%      for j = 2:m
%          clear RSj
%          RSj = ones(1,nr);
%          for k=1:j
%              RSj = RSj + MatRM(k,:);
%          end
%          MatRZ(j,:) = RSj;
%      end
%   end  
MatRZ = MatRM;
                                            %--- Normalisation
  RmeanZ    = mean(MatRZ,2);
  MatRZ0    = MatRZ - repmat(RmeanZ,1,nr);
  MatRcovZ0 = cov(MatRZ0');
  [MatRL]   = chol(MatRcovZ0);
  MatRetagr = (MatRL')\MatRZ0;
%   RmeanHgr   = mean(MatRetagr,2);
%   MatRcovHgr = cov(MatRetagr');
return                                                                        
