 function [ArrayZsim] = sub_integration_Verlet_MC_for_group(nu,m,N_d,M0transient,nbMC,Deltar,f0,MatReta,MatRa,MatRg,shss,sh, ...
                                                               ArrayWiennerM0transient,MatRGauss)  
    %
    % Copyright C. Soize  July 2015, modified 11 June 2016, modified 09 April 2017, modified 09 October 2017,modified 21 November 2017 
    %
    %
    % Solving the ISDE  with the initial condition MatRz(n,m)
    % Time integration scheme is the Verlet algorithm
    %
    %--- INPUTS
    %      nu   = number of rows in MatRZsim 
    %      m    = number of columns in MatRZsim 
    %      N_d  = number of realizations in the database (initial data)
    %      M0transient = number of steps for killng the transient part of the response
    %      M0 = number of steps between two snapshots on a trajectory:  M0*Deltar = duration of a trajectory
    %      nbMC = number of snapshots that are  M0, 2*M0,..., nbMC*M0
    %      Deltar = integration step
    %      f0   = dissipation coefficient
    %      MatReta(nu,N_d)
    %      MatRa(N_d,m)
    %      MatRg(N_d,m)
    %      shss and sh parameters for the kernel density estimation
    %      ArrayWiennerM0transient(nu,N_d,M0transient,nbMC)
    %      MatRGauss(nu,N_d,nbMC)
    %
    %--- OUTPUTS
    %      ArrayZsim(nu,m,nbMC) = [MatRZsim(nu,m)^1 ..... MatRZsim(nu,m)^nbMC)] with MatRZsim(nu,m)^1^ell,  ell = 1,...,nbMC
    
    ArrayZsim = zeros(nu,m,nbMC);           % ArrayZsim(nu,m,nbMC) 
    
    b         = f0*Deltar/4;
    a0        = Deltar/2;
    a1        = (1-b)/(1+b);
    a2        = Deltar/(1+b);
    a3        = (sqrt(f0*Deltar))/(1+b);
    MatRZ0    = MatReta*MatRa;              % MatRZ0(nu,m) , MatRz(nu,m) : set of realizations following the invariant measure
    
                                            %--- realization ell = 1
    MatRZ            = MatRZ0; 
    ArrayZsim(:,:,1) = MatRZ;               %    ArrayZsim(nu,m,nbMC) 
    
    for ell = 2:nbMC                        %--- realizations ell = 2, ... , nbMC
     
        MatRV0 = MatRGauss(:,:,ell)*MatRa;                                 % MatRV0(nu,m) : [independent normalized Gaussian rv]*MatRa, MatRGauss(nu,N_d,nbMC)
        MatRZ  = MatRZ0;
        MatRV  = MatRV0;   
                                                                           % Killing the transient part of the response
        for k=1:M0transient                  
            MatRZ1s2 = MatRZ + a0*MatRV; 
           [MatRLrond1s2] = sub_Lrond(nu,N_d,MatReta,MatRa,MatRg,MatRZ1s2,shss,sh);
            MatRWrond = ArrayWiennerM0transient(:,:,k,ell)*MatRa;          % MatRWrond(nu,m), MatRa(N_d,m), ArrayWiennerM0transient(nu,N_d,M0transient,nbMC)
            MatRVp1 = a1*MatRV + a2*MatRLrond1s2 + a3*MatRWrond;    
            MatRZp1 = MatRZ1s2 + a0*MatRVp1; 
            MatRZ = MatRZp1;                                               % MatRZp1(nu,m)
            MatRV = MatRVp1;
        end 
                                                                           % Loading the realization ell
        ArrayZsim(:,:,ell) = MatRZ;                                        % ArrayZsim(nu,m,nbMC)
    end  
    return
 end

