 function [MatRZsim_ell] = sub_integration_Verlet_MC_for_parallel_c1(ell,nu,m,N_d,M0transient,Deltar,f0,MatReta,MatRa,MatRg,shss,sh, ...
                                                                    ArrayWiennerM0transient_ell,MatRGauss_ell,mhc,Rlambda_iter)  
    %
    % Copyright C. Soize  12 November 2020 
    %
    %--- ind_constraints = 1 : constraints E{H_j^2} = 1 for j =1,...,nu  
    %
    % Solving the ISDE  with the initial condition MatRz(n,m)
    % Time integration scheme is the Verlet algorithm
    %
    %--- INPUTS
    %      nu   = number of rows in MatRZsim 
    %      m    = number of columns in MatRZsim 
    %      N_d  = number of realizations in the database (initial data)
    %      M0transient = number of steps for killng the transient part of the response
    %      M0 = number of steps between two snapshots on a trajectory:  M0*Deltar = duration of a trajectory
    %      nbMC = number of snapshots that are  M0, 2*M0,..., nbMC*M0
    %      Deltar = integration step
    %      f0   = dissipation coefficient
    %      MatReta(nu,N_d)
    %      MatRa(N_d,m)
    %      MatRg(N_d,m)
    %      shss and sh parameters for the kernel density estimation
    %      ArrayWiennerM0transient_ell(nu,N_d,M0transient)
    %      MatRGauss_ell(nu,N_d)
    %      mhc = nu 
    %      Rlambda_iter(mhc,1) = ( lambda_1,...,lambda_nu) , mhc = nu
    %
    %--- OUTPUTS
    %      MatRZsim_ell(nu,m)
    
    b         = f0*Deltar/4;
    a0        = Deltar/2;
    a1        = (1-b)/(1+b);
    a2        = Deltar/(1+b);
    a3        = (sqrt(f0*Deltar))/(1+b);
                                            %--- realizations ell = 1, ... , nbMC
    MatRZ0    = MatReta*MatRa;              %    MatRZ0(nu,m) , MatRz(nu,m) : set of realizations following the invariant measure
    MatRV0 = MatRGauss_ell*MatRa;           %    MatRV0(nu,m) : [independent normalized Gaussian rv]*MatRa, MatRGauss_ell(nu,N_d)
    MatRZ  = MatRZ0;
    MatRV  = MatRV0;                                                  % Killing the transient part of the response
    for k=1:M0transient                  
        MatRZ1s2 = MatRZ + a0*MatRV; 
       [MatRLrond1s2] = sub_Lrond_c1(nu,N_d,MatReta,MatRa,MatRg,MatRZ1s2,shss,sh,mhc,Rlambda_iter);
        MatRWrond = ArrayWiennerM0transient_ell(:,:,k)*MatRa;          % MatRWrond(nu,m), MatRa(N_d,m), ArrayWiennerM0transient_ell(nu,N_d,M0transient)
        MatRVp1 = a1*MatRV + a2*MatRLrond1s2 + a3*MatRWrond;    
        MatRZp1 = MatRZ1s2 + a0*MatRVp1; 
        MatRZ = MatRZp1;                                               % MatRZp1(nu,m)
        MatRV = MatRVp1;
    end                                                                % Loading the realization ell
    MatRZsim_ell = MatRZ;                                              % MatRZsim_ell(nu,m)
    return
 end

