function [Rpdf] = sub_ksdensity_mult(n,nsim,dim,nexp,MatRxData,MatRxExp)

                 % sub_ksdensity_mult(n,nsim,1,RUData,Rpoint)
%
% Copyright C. Soize 23 April 2014 
%
% Cette fonction calcule la valeur, en nexp vecteurs experimentaux (ou scalaires si dim=1) Rx^1,...,Rx^nexp stockes dans 
% le tableau MatRxExp = matrice (dim,nexp), de la fonction de densite de probabilite conjointe d'un 
% vecteur alatoire RX de dimension dim, CONSIDERE COMME LA LOI MARGINALE D'UN VECTEUR ALEATOIRE DE DIMENSION  n >= dim,
% dont les nsim ralisations: RX(theta_1),..., RX(theta_nsim) sont utilises pour estimer la fonction de densit de probabilit 
% marginale par la mthode des noyaux Gaussiens, sont stokes dans le tableau MatRxData = matrice (dim,nsim).
%
%  Rhatsigma(j) = empirical standard deviation of RX(j) estimated with MatRxData(j,:) = [RX(theta_1),..., RX(theta_nsim)]
%  s = [4/(nsim*(2+n))]^(1/(n+4)) : Silverman bandwidth pour le vecteur de dimension n (et non pas ndim)
%  Rsigma(j) = Rhatsigma(j)* s   
%  shss = 1/sqrt(s2+(nsim-1)/nsim);   : coef correcteur : sh et sh/s introduit par C. Soize
%  sh = s*shss;                        
%
%   MatRxExpNorm(j,:)  = MatRxExp(j,:)/Rhatsigma(j);                       
%   MatRxDataNorm(j,:) = MatRxData(j,:)/Rhatsigma(j);    
%
%---INPUT variables
%
% MatRxData = matrice (dim,nsim) des nsim realisations independantes utilises pour estimer la 
%             densit de probabilit du vecteur alatoire RX de dimension dim.
% MatRxExp  = matrice (dim,nexp) des nexp donnes experimentales de Rx o la pdf est estime
%
%---OUTPUT variables
%
%          Rpdf(nexp,1) = valeur de la fonction de densit de probabilit aux points  Rx^1,...,Rx^nexp     

Rhatsigma =(std(MatRxData'))'; 
s = (4/((n+2)*nsim))^(1/(n+4));
s2 = s*s;

shss = 1/sqrt(s2+(nsim-1)/nsim);   % coef sh/s
sh = s*shss;                       % coed sh
sh2 = sh*sh;

cons = 1/(prod(sqrt(2*pi)*sh*Rhatsigma));
MatRxDataNorm = zeros(dim,nsim);                                           
MatRxExpNorm  = zeros(dim,nexp);                                           
for j =1:dim                                                              
    MatRxExpNorm(j,:)  = MatRxExp(j,:)/Rhatsigma(j);                       
    MatRxDataNorm(j,:) = MatRxData(j,:)/Rhatsigma(j);                      
end

Rpdf = zeros(nexp,1);                                                       % Rpdf(nexp,1)
                   %--sequence quasi scalaire
                   % for alpha = 1:nexp    
                   %     for ell=1:nsim                                   
                   %         norm2 = norm(shss*MatRxDataNorm(:,ell)-MatRxExpNorm(:,alpha) , 2);  % MatRxDataNorm(dim,nsim), MatRxExpNorm(dim,nexp)
                   %         MatRS(ell,alpha) = exp(-0.5*norm2^2/sh2);                                       
                   %     end 
                   %     Rpdf(alpha)= cons*sum(MatRS(:,alpha))/nsim;                             % Rpdf(nexp,1)
                   % end 
                   %--sequence equivalente vectorise
MatRbid = zeros(dim,nsim,nexp);                                            % MatRbid(dim,nsim,nexp),
for alpha = 1:nexp    
    MatRbid(:,:,alpha) = shss*MatRxDataNorm(:,:)-repmat(MatRxExpNorm(:,alpha),1,nsim); % MatRbid(dim,nsim,nexp), MatRxDataNorm(dim,nsim), MatRxExpNorm(dim,nexp)
end
MatRexpo = permute(MatRbid,[2 3 1]);                                        % MatRexpo(nsim,nexp,dim) = MatRbid(dim,nsim,nexp)
MatRS = exp(-(sum(MatRexpo.^2,3))/(2*sh2));                                 % MatRS(nsim,nexp) 
for alpha = 1:nexp 
     Rpdf(alpha)= cons*sum(MatRS(:,alpha))/nsim;                            % Rpdf(nexp,1)
end 

return                                                                        
