
function [INDEPgr1gr2] = sub_testINDEPgr1gr2(NKL,nr,MatRHexp,MatRHexpGauss,Indgr1,Indgr2,n1,n2)
%
% Copyright C. Soize 07 January 2016 
%
%    Testing the independence of the random vector  H^gr1 = (H_j1,...,H_jn1) with the random vector H^gr2 = (H_k1,...,H_kn2) 
%    with Indgr1 = (j1,...,jn1) the indices for group 1 and Indgr2 = (k1,...,kn2) the indices for group 2, 
%    using, as the independence criterion, the  MUTUAL INFORMATION.
%
%---INPUT variables
%
%   MatRHexp  = matrice (NKL,nr) des nr ralisations indpendantes du vecteur alatoire H = (H_1,...,H_NKL)
%   n1        = dimension du vecteur H^gr1 = (H_j1,...,H_jn1) (groupe 1)
%   n2        = dimension du vecteur H^gr2 = (H_k1,...,H_kn2) (groupe 2)
%   Indgr1    = vecteur (n1,1) contenant (j1,...,jn1)
%   Indgr2    = vecteur (n2,1) contenant (k1,...,kn2)
%
%---OUTPUT variables
%
%  INDEPgr1gr2 ~  0 if H^gr1 and H^gr2 are independent
%               > 0 if H^gr1 and H^gr2 are dependent
 
                                     % (1) La solution Gauss indpendante est construite pour avoir la rfrence numrique du
                                     %     critere d'indpendance avec le mme nombre de ralisations : nr    
                                     % (2) INDEPgr1gr2 =  Sgr1 + Sgr2 - Sgr1gr2
                                     %                     avec Sgr1 l'entropie de H^gr1, Sgr2 l'entropie de H^gr2 et                                                           
                                     %                     Sgr1gr2 l'entropie de (H^gr1,H^gr2).
                                     %                     Si H^gr1 est indpendante de H^gr2, alors INDEPgr1gr2 = 0
                                     %                     Si H^gr1 et H^gr2 sont dpendantes, alors INDEPgr1gr2 > 0
         MatRHexpgr1 = MatRHexp(Indgr1,:);                            
         MatRHexpgr2 = MatRHexp(Indgr2,:); 
         MatRHexpGaussgr1 = MatRHexpGauss(Indgr1,:);                            
         MatRHexpGaussgr2 = MatRHexpGauss(Indgr2,:);  
                                                                                                  %--- valeurs du log de la pdf de H^gr1 et de la pdf de H^gr2
         [Rlogpdfgr1]      = sub_log_ksdensity_mult(NKL,nr,n1,nr,MatRHexpgr1,MatRHexpgr1);        %    Rlogpdfgr1(nr,1) 
         [Rlogpdfgr2]      = sub_log_ksdensity_mult(NKL,nr,n2,nr,MatRHexpgr2,MatRHexpgr2);        %    Rlogpdfgr2(nr,1) 
         [Rlogpdfgr1Gauss] = sub_log_ksdensity_mult(NKL,nr,n1,nr,MatRHexpGaussgr1,MatRHexpGaussgr1);  
         [Rlogpdfgr2Gauss] = sub_log_ksdensity_mult(NKL,nr,n2,nr,MatRHexpGaussgr2,MatRHexpGaussgr2);  
         
         MatRRHData      = [MatRHexpgr1                                                  %     MatRRHData(n1+n2,nr) 
                            MatRHexpgr2];                                                %     Rlogpdfj1j2(nr,1)
         MatRRHDataGauss = [MatRHexpGaussgr1                                        
                            MatRHexpGaussgr2];                                            
         [Rlogpdfgr1gr2]      = sub_log_ksdensity_mult(NKL,nr,n1+n2,nr,MatRRHData,MatRRHData);            
         [Rlogpdfgr1gr2Gauss] = sub_log_ksdensity_mult(NKL,nr,n1+n2,nr,MatRRHDataGauss,MatRRHDataGauss);  
             
         Sgr1 = - mean(Rlogpdfgr1);                                                       % entropie de H^gr1
         Sgr2 = - mean(Rlogpdfgr2);                                                       % entropie de H^gr2
         Sgr1gr2 = -  mean(Rlogpdfgr1gr2);                                                % entropie de H^gr1 et H^gr2
         INDEPgr1gr2 =  Sgr1 + Sgr2 - Sgr1gr2;
return                                                                        
