
function [INDEPr1r2] = sub_testINDEPr1r2(NKL,nr,MatRHexp,r1,r2)
%
% Copyright C. Soize 11 January 2016 
%
%    Testing the independence of the non-Gaussian normalized random variables H_r1 et H_r2 by using the MUTUAL INFORMATION criterion
%    that is written as INDEPr1r2 = Sr1 + Sr2 - Sr1r2 with Sr1 = entropy of H_r1, Sr2 = entropy of H_r2, Sr1r2 =  entropy of (Hr1,Hr2) 
%    The entropy that is a mathematical expectation is estimated with the Monte Carlo method by using the same realizations that the one used
%    for estimating the pdf by the Gaussian kernel method.
%
%---INPUT variables
%
%   MatRHexp  = matrice (NKL,nr) des nr ralisations indpendantes du vecteur alatoire H = (H_1,...,H_NKL)
%   r1  et r2  = indices pour tester l'indpendance statistique de H_r1 avec H_r2
%
%---OUTPUT variables
%
%  INDEPr1r2 : if INDEPr1r2 ~ 0, H_r1 and H_r2 are independent 
%              if INDEPr1r2 > 0, H_r1 and H_r2 are dependent
%
%               [Rlogpdf] = sub_log_ksdensity_mult(n,nsim,dim,nexp,MatRxData,MatRxExp)
%                                                  RX dimension dim, CONSIDERE COMME LA LOI MARGINALE D'UN VECTEUR ALEATOIRE DE DIMENSION  n >= dim.
%                                                  MatRxData = matrice (dim,nsim) des nsim realisations independantes utilises pour estimer la 
%                                                              densit de probabilit du vecteur alatoire RX de dimension dim.
%                                                  MatRxExp  = matrice (dim,nexp) des nexp donnes experimentales de Rx o la pdf est estime

                                                                                              %--- valeurs du log de la pdf de H_r1 et de la pdf de H_r2
         [Rlogpdfr1]      = sub_log_ksdensity_mult(NKL,nr,1,nr,MatRHexp(r1,:),MatRHexp(r1,:));           %    Rlogpdfr1(nr,1) 
         [Rlogpdfr2]      = sub_log_ksdensity_mult(NKL,nr,1,nr,MatRHexp(r2,:),MatRHexp(r2,:));           %    Rlogpdfr2(nr,1) 

         MatRRHData  = [MatRHexp(r1,:)                                                  %     MatRRHData(2,nr) : 2 composantes H_r1 et H_r2
                        MatRHexp(r2,:)];                                                %     Rlogpdfr1r2(nr,1)
                                            
         [Rlogpdfr1r2]      = sub_log_ksdensity_mult(NKL,nr,2,nr,MatRRHData,MatRRHData);            
                    
         Sr1 = - mean(Rlogpdfr1);                                                       % entropy of H_r1
         Sr2 = - mean(Rlogpdfr2);                                                       % entropy of H_r2
         Sr1r2 = -  mean(Rlogpdfr1r2);                                                  % entropy of (H_r1,Hr2)
         INDEPr1r2 = Sr1 + Sr2 -  Sr1r2;
         
return                                                                        
