
function [INDEPr1r2Gauss] = sub_testINDEPr1r2GAUSS(NKL,nr,MatRHexpGauss,r1,r2)
%
% Copyright C. Soize 07 January 2016 
%
%    Numerical criterion for testing the independence of two normalized Gaussian random variables HGauss_r1 and HGauss by using 
%    the MUTUAL INFORMATION criterion: INDEPr1r2Gauss = S(HGauss_r1) + S(HGauss_r2)- S(HGauss_r1,HGauss_r2), that is positive or equal to zero, in which
%    S(HGauss_r1), S(HGauss_r2), and S(HGauss_r1,HGauss_r2) are the entropies. Comme HGauss_r1 and HGauss_r2 are normalized random variables, we should have
%    INDEPr1r2Gauss  = 0. However as the entropy are estimated with the Monte Carlo method, we have INDEPr1r2Gauss > 0 (and not = 0).
%    This positive value is thus used as a numerical reference for testing the independence of the non-Gaussian normalized random variables H_r1 and H_r2
%
%---INPUT variables
%
%   MatRHexpGauss  = matrice (NKL,nr) des nr ralisations indpendantes du vecteur alatoire HGauss = (HGauss_1,...,HGauss_NKL)
%   r1  et r2  = indices pour valuer le critre d'indpendance statistique de HGauss_r1 avec HGauss_r2
%
%---OUTPUT variables
%
%   INDEPr1r2Gauss (thoriquement INDEPr1r2Gauss = 0 pour tout r1 et r2)
%
%    (1) La solution Gauss indpendante est construite pour avoir la rfrence numrique du
%        critere d'indpendance avec le mme nombre de ralisations : nr    
%    (2) INDEPr1r2Gauss : pour chaque couple icouple=(r1,r2), INDEPr1r2Gauss = Sr1Gauss + Sr2Gauss -  Sr1r2Gauss
%        avec Sr1Gauss l'entropie de HGauss_r1, Sr2Gauss l'entropie de HGauss_r2 et                                                           
%             Sr1r2Gauss l'entropie conjointe de HGauss_r1 et HGauss_r2.              
         
         [Rlogpdfr1Gauss] = sub_log_ksdensity_mult(NKL,nr,1,nr,MatRHexpGauss(r1,:),MatRHexpGauss(r1,:));  
         [Rlogpdfr2Gauss] = sub_log_ksdensity_mult(NKL,nr,1,nr,MatRHexpGauss(r2,:),MatRHexpGauss(r2,:));  
         MatRRHDataGauss = [MatRHexpGauss(r1,:)                                        
                            MatRHexpGauss(r2,:)];  
         [Rlogpdfr1r2Gauss] = sub_log_ksdensity_mult(NKL,nr,2,nr,MatRRHDataGauss,MatRRHDataGauss);  
         Sr1Gauss = - mean(Rlogpdfr1Gauss);                                                       
         Sr2Gauss = - mean(Rlogpdfr2Gauss);                                                       
         Sr1r2Gauss = -  mean(Rlogpdfr1r2Gauss);                                                  
         INDEPr1r2Gauss = Sr1Gauss + Sr2Gauss - Sr1r2Gauss;     % devrait tre gale  0 et correspond  l'indpendance
return                                                                        
